/*****************************************************************************/
/*!
\file      uart.c
\brief     driver and handling functions for UART0/1              

\implementation
  project       Multimedia Gateway
  version       0.01
  copyright     
  
  generated     Matthias Koelling 01.03.2007
  review
  released

  compiler      AVR GCC
  target        ATmega128-16AU
\endimplementation

\history  
  Rev     Date         Author      Commentary
  0.00    01.03.2007   Koelling    generated
  0.01    15.03.2008   Koelling    replaced printstrfunctions
\endhistory  
*/
/*****************************************************************************/


/* -- Includes ------------------------------------------------------------- */

#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <stdlib.h>
#include "main.h"
#include "hardware.h"
#include "com.h"


/* -- Configuration Parameters --------------------------------------------- */

/* -- Makros --------------------------------------------------------------- */

/* -- Defines -------------------------------------------------------------- */ 

#define BAUD_CONTROL_DU32    (uint32_t) 19200
#define BAUD_DEBUG_DU32      (uint32_t)115200
#define TX_BUFFER_END        12

/* -- Types ---------------------------------------------------------------- */

/* -- Module Global Variables ---------------------------------------------- */

static uint8_t rx_read_ptr_u8;
static uint8_t rx_write_ptr_u8;

static uint8_t tx_read_ptr_u8;
static uint8_t tx_write_ptr_u8;

static uint8_t txinptr;            
static uint8_t txoutptr;       
static uint8_t const  *txbuffer[TX_BUFFER_END]; // 39-26-1 = 12 serial output strings queue   

//----------------------------------------------------------------------
// Some simple UART IO functions.
//----------------------------------------------------------------------

// UART receive interrupt.  Fetch the character received and buffer
// it, unless there was a framing error.  Note that the main loop
// checks the received character only once per 10 ms.

const uint8_t sHEX[]      PROGMEM = {'0',0,'1',0,'2',0,'3',0,'4',0,'5',0,'6',0,'7',0,'8',0,'9',0,'A',0,'B',0,'C',0,'D',0,'E',0,'F',0};

volatile uint8_t rx_int0 = 0;
volatile uint8_t rx_int1 = 0;
volatile uint8_t rxbuf0[RX_BUFFER_LEN];
volatile uint8_t rxbuf1[RX_BUFFER_LEN];
volatile static uint8_t txbuf0[TX_BUFFER_LEN];
volatile static uint8_t txbuf1[TX_BUFFER_LEN];

//-----------------------------------------------------------------------------
/*!
   \brief     ISR(USART0_UDRE_vect)

    transmit interrupt for UART0

   \author     Koelling
   \date       20.10.2008

   \param[in]  void
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

ISR(USART0_UDRE_vect)
{
   UDR0 = txbuf0[tx_read_ptr_u8];
   tx_read_ptr_u8++;
   if(tx_read_ptr_u8 == TX_BUFFER_LEN)
   {
      tx_read_ptr_u8 = 0;
   }
   if(tx_read_ptr_u8 == tx_write_ptr_u8)
   {
      UCSR0B &= ~_BV(UDRIE0);
   }
}


//-----------------------------------------------------------------------------
/*!
   \brief     ISR(USART0_RX_vect)

    receive interrupt for UART0

   \author     Koelling
   \date       01.03.2007

   \param[in]  void
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

ISR(USART0_RX_vect)
{
   uint8_t c;

   c = UDR0;

   if(bit_is_clear(UCSR0A, FE0))
   {
      rxbuf0[rx_write_ptr_u8] = c;
      rx_write_ptr_u8++;
      if(rx_write_ptr_u8 == RX_BUFFER_LEN)
      { 
      	rx_write_ptr_u8 = 0;
      }   
   }
}


//-----------------------------------------------------------------------------
/*!
   \brief     ISR(USART1_RX_vect)

    receive interrupt for UART1

   \author     Koelling
   \date       01.03.2007

   \param[in]  void
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

ISR(USART1_RX_vect)
{
   uint8_t c;
   static uint8_t i;

   c = UDR1;

   if (bit_is_clear(UCSR1A, FE1))
   {
      if(rx_int1 == 0)
      {
         if((i < (RX_BUFFER_LEN - 1)) && (c != '\r'))
         { 
            rxbuf1[i] = c;
            i++;
         }
         else
         {
            rxbuf1[i] = 0;
            rx_int1 = 1;
            i = 0;
         }
      }
   }
}


//-----------------------------------------------------------------------------
/*!
   \brief     void Init_UART(void)

    initialization of UART0 and UART1
    initialization of variables

   \author     Koelling
   \date       01.03.2007

   \param[in]  void
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

void Init_UART(void)
{
#ifdef SIMU
   uint8_t i = 0;
#endif
   
   UCSR0A = _BV(U2X0);		                            // improves baud rate error
   UCSR0B = _BV(TXEN0)|_BV(RXEN0)|_BV(RXCIE0);         // tx/rx enable, rx complete intr 
   UCSR0C = _BV(UCSZ01)|_BV(UCSZ00);                   // 8 databits 
   UBRR0L = (F_CPU / (BAUD_CONTROL_DU32 * 8)) - 1;     // calculate baudrate and set low byte
   UBRR0H = (F_CPU / (BAUD_CONTROL_DU32 * 8) - 1) >> 8;// calculate baudrate and set high byte

   UCSR1A = _BV(U2X1);		                            // improves baud rate error
   UCSR1B = _BV(TXEN1)|_BV(RXEN1)|_BV(RXCIE1);         // tx/rx enable, rx complete intr 
   UCSR1C = _BV(UCSZ11)|_BV(UCSZ10);                   // 8 databits
   UBRR1L = (F_CPU / (BAUD_DEBUG_DU32 * 8)) - 1;       // calculate baudrate and set low byte
   UBRR1H = (F_CPU / (BAUD_DEBUG_DU32 * 8) - 1) >> 8;  // calculate baudrate and set high byte
   
   WR = TRUE;                                          // write enable FTDI idle  
   txinptr = 0;                                        // queue pointers
   txoutptr = 0;

#ifdef SIMU
   rxbuf0[i] = 0x5c;
   i++;
   rxbuf0[i] = 0x48;
   i++;
   rxbuf0[i] = 'F';
   i++;
   rxbuf0[i] = 'N';
   i++;
   rxbuf0[i] = 'E';
   i++;
   rxbuf0[i] = 'X';
   i++;
   rxbuf0[i] = 'T';
   i++;
   rxbuf0[i] = ' ';
   i++;
   rxbuf0[i] = ' ';
   i++;
   rxbuf0[i] = ' ';
   i++;
   rxbuf0[i] = ' ';
   i++;
   rxbuf0[i] = ' ';
   i++;
   rxbuf0[i] = ' ';
   i++;
   rxbuf0[i] = ' ';
   i++;
   rxbuf0[i] = ' ';
   i++;
   
   for(i;i < (RX_BUFFER_LEN- 3);i++)
	{
      rxbuf0[i] = 0x32;
   }
   /*
   rxbuf0[i] = 0x08;
   i++;
   */
#endif
   tx_read_ptr_u8 = 0;
   tx_write_ptr_u8 = 0;
   rx_read_ptr_u8 = 0;
#ifdef SIMU
   rx_write_ptr_u8 = RX_BUFFER_LEN;
#else
   rx_write_ptr_u8 = 0;
#endif
}


// Send character c down the UART Tx, wait until tx holding register
// is empty.
/*
void Send_Char_fv(char c, uint8_t device_u8)
{
   if(device_u8 == CONTROL_PC)
   {
      loop_until_bit_is_set(UCSR1A, UDRE1);
      UDR1 = c;
   }
   else
   {
      if(device_u8 == CONTROL_MP3)
      {
         loop_until_bit_is_set(UCSR0A, UDRE0);
         UDR0 = c;
      }
   }
}
*/

uint8_t getchr(uint8_t *buffer_u8)
{
	uint8_t length_u8;
   
   length_u8 = 0;
   
   while(rx_write_ptr_u8 != rx_read_ptr_u8)
	{
		*buffer_u8 = rxbuf0[rx_read_ptr_u8];
		buffer_u8++;
      rx_read_ptr_u8++;
		length_u8++;
      #ifndef SIMU
         if(rx_read_ptr_u8 == RX_BUFFER_LEN) rx_read_ptr_u8 = 0; 
	   #endif
   }
	#ifdef SIMU
      if(rx_read_ptr_u8 == RX_BUFFER_LEN) rx_read_ptr_u8 = 0;
   #endif
   return length_u8;	
}


//-----------------------------------------------------------------------------
/*!
   \brief     void Send_Char_fv(char c, uint8_t device_u8)

    put the character to send in the corresponding tx register

   \author     Koelling
   \date       01.03.2007

   \param[in]  uint8_t c         -> character to send
               uint8_t device_u8 -> UART0 / UART1
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------
/*
void Send_Char_fv(uint8_t char_u8, uint8_t device_u8)
{
   if(device_u8 == CONTROL_PC)
   {
      #ifndef SIMU   
         loop_until_bit_is_set(UCSR1A, UDRE1);
      #endif   
      UDR1 = char_u8;
   }
   else
   {
      if(device_u8 == CONTROL_MP3)
      {
         txbuf0[tx_write_ptr_u8] = char_u8;
         tx_write_ptr_u8++;
         if(tx_write_ptr_u8 == TX_BUFFER_LEN)
         {
            tx_write_ptr_u8 = 0;
         } 
         UCSR0B |= _BV(UDRIE0);
      }
   }
}
*/

void Send_Char_fv(uint8_t char_u8, uint8_t device_u8)
{
   switch(device_u8)
   {
   case KLINE:
      #ifndef SIMU   
         loop_until_bit_is_set(UCSR1A, UDRE1);
      #endif   
      UDR1 = char_u8;
      break;
   case CONTROL_MP3:
      txbuf0[tx_write_ptr_u8] = char_u8;
      tx_write_ptr_u8++;
      if(tx_write_ptr_u8 == TX_BUFFER_LEN)
      {
         tx_write_ptr_u8 = 0;
      } 
      UCSR0B |= _BV(UDRIE0);
      break;
   case CONTROL_PC:
      FTDI_OUT(char_u8);
      WR = FALSE;
      WR = TRUE;
      break;
   default:
      break;
   }
}

//-----------------------------------------------------------------------------
/*!
   \brief     void Send_Data_String_fv(const uint8_t *string_pau8, 
                                       uint8_t device_u8)

    Send a 0-terminated string down the UARTx specified by the parameter 
    device_u8. The start address of the string is located in data memory.

   \author     Koelling
   \date       01.03.2007

   \param[in]  *string_pau8      -> start address of the string
               uint8_t device_u8 -> UART0 / UART1
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

void Send_Data_String_fv(const char *string_pau8, uint8_t device_u8)
{
   while(*string_pau8 != 0)
   {
	   Send_Char_fv(*string_pau8, device_u8);
      string_pau8++;
   }
}

   
//-----------------------------------------------------------------------------
/*!
   \brief     void Send_Const_String_fv(const uint8_t *string_pau8, 
                                        uint8_t device_u8)

    Send a 0-terminated string down the UARTx specified by the parameter 
    device_u8. The start address of the string is located in program
    memory.

   \author     Koelling
   \date       01.03.2007

   \param[in]  *string_pau8      -> start address of the string
               uint8_t device_u8 -> UART0 / UART1
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

void Send_Const_String_fv(const char *string_pau8, uint8_t device_u8)
{
   uint8_t char_u8;

   char_u8 = pgm_read_byte(string_pau8);
   while(char_u8 != 0)
   {
	   Send_Char_fv(char_u8, device_u8);
      string_pau8++;
      char_u8 = pgm_read_byte(string_pau8);
   }
}

//-----------------------------------------------------------------------------
/*!
   \brief     void EnqueueString(const uint8_t addr PROGMEM)

    EnqueueString - Adds a new string pointer into the outgoing serial string
    queue.  
     
   \author     Koelling
   \date       02.10.2007

   \param[in]  const uint8_t addr PROGMEM -> start address of string to display
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

void EnqueueString(const uint8_t *addr PROGMEM)
{
   txbuffer[txinptr] = addr;
   txinptr++;
   if(txinptr == TX_BUFFER_END)
   {
      txinptr = 0;
   }
}

//-----------------------------------------------------------------------------
/*!
   \brief     void EnqueueHex(uint8_t hexbyte_u8)

    The byte is converted to a two byte ASCII hexidecimal string
     
   \author     Koelling
   \date       05.10.2007

   \param[in]  uint8_t hexbyte   -> hexbyte to display
               uint8_t device_u8 -> UART0 / UART1        
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

void EnqueueHex(uint8_t hexbyte_u8, uint8_t device_u8)
{
   uint8_t nibble_u8;

   nibble_u8 = hexbyte_u8 >> 4;      // send high nibble first
   nibble_u8 <<= 1;                  // multiply high nibble by 2
   Send_Const_String_fv(&sHEX[nibble_u8], device_u8);

   nibble_u8 = hexbyte_u8 & 0x0F;	 // prepare low nibble
   nibble_u8 <<= 1;                  // multiply low nibble by 2
   Send_Const_String_fv(&sHEX[nibble_u8], device_u8);
}

// eof
