/*****************************************************************************/
/*!
\file      device.c
\brief     stuff related to audio device selection and the required action
           for cdc emulation 
                         

\implementation
  project       Multimedia Gateway
  version       0.01
  copyright     Matthias Koelling 
  
  generated     Matthias Koelling 19.06.2008
  review
  released

  compiler      AVR GCC
  target        ATmega1280-16AU
\endimplementation

\history  
  Rev     Date         Author      Commentary
  0.01    27.04.2008   Koelling    adaptaption for gateway4
  0.00    19.06.2008   Koelling    generated
\endhistory  
*/
/*****************************************************************************/

/* -- Includes ------------------------------------------------------------- */

#include <avr/io.h>
#include <stdlib.h>
#include <string.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include "hardware.h"
#include "stv5730.h"
#include "tda7439.h"
#include "main.h"
#include "vwcdc.h"
#include "device.h"
#include "display.h"

/* -- Configuration Parameters --------------------------------------------- */

/* -- Defines -------------------------------------------------------------- */ 

#define CDC     0
#define RADIO   0
#define GATEWAY 1

// to increase the write cycles we use an array in EEPROM for the last device 
// information, when the access counter reaches its maximum the next place in 
// array is used

#define EE_ARRAY_SIZE 8 
                    //   

// bit length definition PWM command
#define HIGHBIT_LOW     3400     // 1700us
#define LOWBIT_LOW      1100     // 550us
#define STARTBIT_LOW    9000     // 4500us
//#define STARTBIT_LOW    10     
#define BIT_HIGH        1100     // 550us
#define INITHIGH       18000     // 9ms
#define LAST_LOW_TIME  40000     // 20ms

/* -- Makros --------------------------------------------------------------- */

#define SEL_COMMAND_CDC(source) (MUX_A = source)
#define SEL_DATA_RADIO(source)  (MUX_B = source)
#define SEL_CLK_RADIO(source)   (MUX_C = source)

/*
#define SEL_AUDIO_MP3 { CDC_MP3 = TRUE;\
                        AUX_CDC_MP3 = TRUE;\
                      }   

#define SEL_AUDIO_CDC { CDC_MP3 = FALSE;\
                        AUX_CDC_MP3 = TRUE;\
                      }   

#define SEL_AUDIO_AUX { CDC_MP3 = TRUE;\
                        AUX_CDC_MP3 = FALSE;\
                      }   
*/

#define SEL_AUDIO_MP3  Sel_Audio_MP3_fv()

#define SEL_AUDIO_CDC  Sel_Audio_CDC_fv()  

#define SEL_AUDIO_AUX  Sel_Audio_AUX_fv()  


enum PWM_COMMAND_STATES 
{
   START_LOW = 1,		
   START_HIGH,		
   BYTE_LOW,
   BYTE_HIGH,
   LAST_LOW,
   STOP
};

/* -- Types ---------------------------------------------------------------- */

/*
typedef enum
{
   M_ENABLE,		
   M_DISABLE,		
   NUM_PWM_COMMANDS
}PWM_COMMANDS_E;
*/
typedef struct
{
   uint16_t access_counter_u16;
   DEVICE_E last_device_e;
}EE_DEVICE_T;

/* -- Modul Global Function Prototypes ------------------------------------- */

//static void Send_Gateway_Command_fv(PWM_COMMANDS_E pwm_command_e);

/* -- Module Global Variables ---------------------------------------------- */

// EEPROM variables
uint8_t ee_array_index_mu8 __attribute__((section(".eeprom")));
EE_DEVICE_T ee_device_mat[EE_ARRAY_SIZE] __attribute__((section(".eeprom")));

// RAM variables
static uint16_t access_counter_mu16;
static uint8_t array_index_mu8;
static uint8_t enable_IR_select_mu8 = TRUE;
static DEVICE_E last_device_me;
static DEVICE_E current_device_me;
static uint8_t switch_on_mu8 = FALSE;

static uint8_t pwm_command_state_mu8;
static uint8_t *pwm_command_mpu8;
static const uint8_t pwm_command_cau8[NUM_PWM_COMMANDS][4] PROGMEM = {
                                                                       {0x53,0x2C,0x08,0xF7}, // M_ENABLE
                                                                       {0x53,0x2C,0x10,0xEF}  // M_DISABLE
                                                                     };

/* -- Extern Global Variables ---------------------------------------------- */

/* -- Implementation Functions --------------------------------------------- */


ISR(TIMER1_COMPC_vect)
{
   static uint8_t pwm_bit_cnt_hu8 = 0;
   static uint8_t pwm_byte_cnt_hu8 = 0;
   static uint8_t pwm_byte_u8;
   switch(pwm_command_state_mu8)
   {
   case START_LOW:
      COMMAND_GATEWAY = OFF;
      OCR1C += STARTBIT_LOW;
      pwm_command_state_mu8 = START_HIGH;
      break;
   case START_HIGH:
      COMMAND_GATEWAY = ON;
      OCR1C += BIT_HIGH;
      pwm_byte_u8 = pgm_read_byte(pwm_command_mpu8);
      pwm_command_mpu8++;
      pwm_command_state_mu8 = BYTE_LOW;
      break;
   case BYTE_LOW:
      COMMAND_GATEWAY = OFF;
      if((pwm_byte_u8 & 0x80) == 0)
      {
         OCR1C += LOWBIT_LOW;
      }
      else
      {
         OCR1C += HIGHBIT_LOW;
      }
      pwm_byte_u8 <<= 1;
      pwm_command_state_mu8 = BYTE_HIGH;
      break;
   case BYTE_HIGH:
      COMMAND_GATEWAY = ON;
      OCR1C += BIT_HIGH;
      pwm_bit_cnt_hu8 ++;
      if(pwm_bit_cnt_hu8 == 8)
      {
         pwm_bit_cnt_hu8 = 0;
         pwm_byte_cnt_hu8++;
         if(pwm_byte_cnt_hu8 == 4)
         {
            pwm_byte_cnt_hu8 = 0;
            pwm_command_state_mu8 = LAST_LOW;
         }
         else
         {
            pwm_byte_u8 = pgm_read_byte(pwm_command_mpu8);
            pwm_command_mpu8++;
            pwm_command_state_mu8 = BYTE_LOW;
         }        
      }
      else
      {
         pwm_command_state_mu8 = BYTE_LOW;
      }
      break;
   case LAST_LOW:
      COMMAND_GATEWAY = OFF;   // set pwm command line to idle state
      OCR1C += LAST_LOW_TIME;
      pwm_command_state_mu8 = STOP;
      break;  
   case STOP:
   default:
      TIFR1 = OCF1C;
      TIMSK1 &= ~_BV(OCIE1C);  // disable output compare interrupt OC1C
      SEL_COMMAND_CDC(RADIO);  // CDC gets further commands from radio
      break;
   }
}


//-----------------------------------------------------------------------------
/*!
   \brief     void Init_Select_fv(void)

    initialisations for audio device selection and switch to the last selected
    device

   \author     Matthias Koelling
   \date       19.06.2008

   \param[in]  void
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

void Init_Select_fv(void)
{
   array_index_mu8 = eeprom_read_byte(&ee_array_index_mu8);
   if(array_index_mu8 == 0xFF)      // first access to EEPROM   
   {
      array_index_mu8 = 0;
      eeprom_write_byte(&ee_array_index_mu8, array_index_mu8);
   }
   if(array_index_mu8 < EE_ARRAY_SIZE)
   {
      access_counter_mu16 = eeprom_read_word(&ee_device_mat[array_index_mu8].access_counter_u16);
      last_device_me = (DEVICE_E)eeprom_read_byte(&(ee_device_mat[array_index_mu8].last_device_e));
   }
   if(last_device_me >= NUM_DEVICES)
   {
      last_device_me = D_MP3;      // default value is MP3 select
   }
   current_device_me = last_device_me; 

   switch(current_device_me)
   {
   case D_MP3:
      // select control lines
      SEL_COMMAND_CDC(GATEWAY);    // CDC gets command from gateway
      SEL_DATA_RADIO(GATEWAY);     // radio gets data from gateway   
      SEL_CLK_RADIO(GATEWAY);      // radio gets clock from gateway
      // select audio lines
      SEL_AUDIO_MP3;
      LED_MP3 = ON;
      break;
   case D_CDC:
      // select control lines
      SEL_COMMAND_CDC(RADIO);      // CDC gets command from radio
      SEL_DATA_RADIO(CDC);         // radio gets data from cdc   
      SEL_CLK_RADIO(CDC);          // radio gets clock from cdc
      // select audio lines
      SEL_AUDIO_CDC;
      LED_CDC = ON;
      //Switch_Display_fv(OFF);
      break;
   case D_AUX:
      // select control lines
      SEL_COMMAND_CDC(GATEWAY);    // CDC gets command from gateway
      SEL_DATA_RADIO(GATEWAY);     // radio gets data from gateway   
      SEL_CLK_RADIO(GATEWAY);      // radio gets clock from gateway
      // select audio lines
      SEL_AUDIO_AUX;
      LED_AUX = ON; 
      Switch_Display_fv(OFF);
     break;
   default:
      // default initialisation enables MP3
      SEL_COMMAND_CDC(GATEWAY);    // CDC gets command from gateway
      SEL_DATA_RADIO(GATEWAY);     // radio gets data from gateway   
      SEL_CLK_RADIO(GATEWAY);      // radio gets clock from gateway
      // select audio lines
      SEL_AUDIO_MP3;
      LED_MP3 = ON;
      //Switch_Display_fv(ON);
      break;
   }
   Display_Active_Device_fv(current_device_me);
}


//-----------------------------------------------------------------------------
/*!
   \brief     void Change_Device_fv(void)

    this function is called when the source button on IR is pressed 

   \author     Matthias Koelling
   \date       22.06.2008

   \param[in]  void
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

void Change_Device_fv(void)
{
   if(enable_IR_select_mu8 == TRUE)
   {
      if(switch_on_mu8 == FALSE)
      {
         switch_on_mu8 = TRUE;
      }
      else
      {
         current_device_me++;
         if(current_device_me == NUM_DEVICES)
         {
            current_device_me = D_MP3;         
         }
      }
      Display_Device_Select_fv(current_device_me, TRUE);
   }
}


//-----------------------------------------------------------------------------
/*!
   \brief     void Select_Device_fv(void)

    this function is called when the sel button on IR is pressed 

   \author     Matthias Koelling
   \date       22.06.2008

   \param[in]  void
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

void Select_Device_fv(void)
{
   if(enable_IR_select_mu8 == TRUE)
   {
      if(current_device_me != last_device_me)
      {
         switch(last_device_me)
         {
         case D_MP3:
            LED_MP3 = OFF;
            Switch_Display_fv(OFF);
            if(current_device_me == D_CDC)
            {
               ENABLE_CDC = ON;
               
               // select control lines
               // SEL_COMMAND_CDC(GATEWAY);       -> done in Send_Gateway_Command_fv 
               // after sending command it switches back to RADIO
               SEL_DATA_RADIO(CDC);               // radio gets data from cdc   
               SEL_CLK_RADIO(CDC);                // radio gets clock from cdc
               // enable device
               Send_Gateway_Command_fv(M_ENABLE); // enable CDC
               SetStateIdle();                    // set statemachine for radio display update
               ENABLE_MP3 = OFF;                  // switch off MP3
               E_INT_MP3 = OFF;
               // select audio lines
               SEL_AUDIO_CDC;                     // audio comes from CDC
               LED_CDC = ON;
            }
            else
            {
               // select control lines            -> nothing to do
               // enable device                   -> nothing to enable
               // select audio lines
               SEL_AUDIO_AUX;                     // audio comes from AUX
               LED_AUX = ON;
            }
            break;
         case D_CDC:
            LED_CDC = OFF;
            ENABLE_CDC = OFF;
               
            // select control lines
            SEL_COMMAND_CDC(GATEWAY);             // CDC gets command from gateway
            SEL_DATA_RADIO(GATEWAY);              // radio gets data from gateway   
            SEL_CLK_RADIO(GATEWAY);               // radio gets clock from gateway
            // enable device
            Send_Gateway_Command_fv(M_DISABLE);   // disable CDC
            SetStateInitPlay();                   // set statemachine for radio display update
            if(current_device_me == D_MP3)
            {
               ENABLE_MP3 = ON;                   // switch on MP3
               E_INT_MP3 = ON;
               // select audio lines
               SEL_AUDIO_MP3;                     // audio comes from MP3
               LED_MP3 = ON;
               Switch_Display_fv(ON);
            }
            else
            {
               // select audio lines
               SEL_AUDIO_AUX;                     // audio comes from AUX
               LED_AUX = ON;
            }
            break;
         case D_AUX:
            LED_AUX = OFF;
            if(current_device_me == D_MP3)
            {
               // select control lines            -> nothing to do
               // enable device
               ENABLE_MP3 = ON;                   // switch on MP3
               E_INT_MP3 = ON;
               // select audio lines
               SEL_AUDIO_MP3;
               LED_MP3 = ON;
               Switch_Display_fv(ON);
            }
            else
            {
               ENABLE_CDC = ON;
               
               // select control lines
               SEL_COMMAND_CDC(GATEWAY);          // CDC gets command from gateway
               SEL_DATA_RADIO(CDC);               // radio gets data from cdc   
               SEL_CLK_RADIO(CDC);                // radio gets clock from cdc
               // enable device
               Send_Gateway_Command_fv(M_ENABLE); // enable CDC
               SetStateIdle();                    // set statemachine for radio display update
               // select audio lines
               SEL_AUDIO_CDC;                     // audio comes from CDC
               LED_CDC = ON;
            }   
            break;
         default:
            break;      
         }
      }
      switch_on_mu8 = FALSE;
      Display_Device_Select_fv(current_device_me, FALSE);
      Display_Active_Device_fv(current_device_me);
      last_device_me = current_device_me;
      if(access_counter_mu16 == 0)
      {
         if(array_index_mu8 < EE_ARRAY_SIZE)
         {
            array_index_mu8++;
         }
         eeprom_write_byte(&ee_array_index_mu8, array_index_mu8);
      }
      else
      {
         access_counter_mu16--;
         eeprom_write_word(&ee_device_mat[array_index_mu8].access_counter_u16, access_counter_mu16);
      }
      eeprom_write_byte(&ee_device_mat[array_index_mu8].last_device_e, (uint8_t)last_device_me);
   }
}


//-----------------------------------------------------------------------------
/*!
   \brief     DEVICE_E Get_Last_Device_fe(void)

    this function returns the last active device 

   \author     Matthias Koelling
   \date       22.06.2008

   \param[in]  void
   \param[out] none
   \return     last_device_me -> last active device
*/
//-----------------------------------------------------------------------------

DEVICE_E Get_Last_Device_fe(void)
{
   return last_device_me;
}


//-----------------------------------------------------------------------------
/*!
   \brief     void Enable_IR_Select_fv(void)

    this function returns the last active device 

   \author     Matthias Koelling
   \date       02.07.2008

   \param[in]  void
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

void Enable_IR_Select_fv(void)
{
   //if(current_device_me == D_MP3)
   {
      Switch_Display_fv(ON);
   }
   enable_IR_select_mu8 = TRUE;
}

//-----------------------------------------------------------------------------
/*!
   \brief     void Disable_IR_Select_fv(void)

    this function returns the last active device 

   \author     Matthias Koelling
   \date       02.07.2008

   \param[in]  void
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

void Disable_IR_Select_fv(void)
{
   Switch_Display_fv(OFF);
   //enable_IR_select_mu8 = FALSE;
}

//-----------------------------------------------------------------------------
/*!
   \brief     void Send_Gateway_Command_fv(void)

    This function initiates the emulation of a PWM cdc command, normally sent 
    by the head unit. When we change the audio device by the IR control it 
    looks like a pressed MODE button on the head unit

   \author     Koelling
   \date       23.06.2008

   \param[in]  none
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

//static 
void Send_Gateway_Command_fv(PWM_COMMANDS_E pwm_command_e)
{
   //static uint8_t pwm_byte_u8;
   SEL_COMMAND_CDC(GATEWAY);    // CDC gets command from gateway
   COMMAND_GATEWAY = TRUE;      // start pwm command with high level
   OCR1C = TCNT1 + INITHIGH;
   TIFR1 |= _BV(OCF1C);
   TIMSK1 |= _BV(OCIE1C);       // enable output compare interrupt OC1C
      pwm_command_mpu8 = &pwm_command_cau8[pwm_command_e][0];
   pwm_command_state_mu8 = START_LOW;
   //pwm_byte_u8 = pgm_read_byte(pwm_command_mpu8);

}


//-----------------------------------------------------------------------------
/*!
   \brief     void Device_Timer_CB_fv(void)(void)

    timer callback function, called when a privious set timing event is over

   \author     Matthias Koelling
   \date       22.06.2008

   \param[in]  void
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

void Device_Timer_CB_fv(void)
{
}

// eof //
