/*****************************************************************************/
/*!
\file      display.c
\brief     this is reengineered code from firmware of display board JW-002
           used with the PJRC MP3 Player:   http://www.pjrc.com/tech/mp3/
           The code was translated from assembler in to C code,
           runs on ATMEGA128 instead of 80c31 and serves STV5730 OSD display
           controller instead of LCD              

\implementation
  project       Multimedia Gateway
  version       0.00
  copyright     ported from 80c31 on display board JW-002 
  
  generated     Matthias Koelling 26.09.2007
  review
  released

  compiler      AVR GCC
  target        ATmega128-16AU
\endimplementation

\history  
  Rev     Date         Author      Commentary
  0.00    26.09.2007   Koelling    generated
\endhistory  
*/
/*****************************************************************************/


/* -- Includes ------------------------------------------------------------- */

#include <avr/io.h>
#include <avr/sfr_defs.h>
#include <avr/eeprom.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "main.h"
#include "com.h"
#include "stv5730.h"
#include "ir.h"
#include "device.h"
#include "hardware.h"
#include "display.h"

/* -- Configuration Parameters --------------------------------------------- */

/* -- Makros --------------------------------------------------------------- */

/* -- Defines -------------------------------------------------------------- */ 

#define TAB_WIDTH       4
#define TAB_WIDTH_MASK  (TAB_WIDTH - 1)


#define FONT0  (DATA | BE(1) | RGB(7) | BK(0)) // font0 background enabled, color white, no blink
#define FONT1  (DATA | BE(1) | RGB(4) | BK(0)) // font1 background enabled, color red, no blink
#define FONT2  (DATA | BE(1) | RGB(1) | BK(0)) // font2 background enabled, color blue, no blink
#define FONT3  (DATA | BE(1) | RGB(2) | BK(0)) // font3 background enabled, color green, no blink
#define FONT4  (DATA | BE(1) | RGB(3) | BK(0)) // font4 background enabled, color cyan, no blink
#define FONT5  (DATA | BE(1) | RGB(5) | BK(0)) // font5 background enabled, color magenta, no blink
#define FONT6  (DATA | BE(1) | RGB(6) | BK(0)) // font6 background enabled, color yellow, no blink
#define FONT7  (DATA | BE(0) | RGB(4) | BK(0)) // font7 background disabled, color red, no blink
#define FONT8  (DATA | BE(0) | RGB(2) | BK(0)) // font8 background disabled, color green, no blink
#define FONT9  (DATA | BE(1) | RGB(4) | BK(0)) // font9 background enabled, color red, no blink
#define FONT10 (DATA | BE(1) | RGB(4) | BK(0)) // font10 background enabled, color red, no blink
#define FONT11 (DATA | BE(1) | RGB(4) | BK(0)) // font11 background enabled, color red, no blink
#define FONT12 (DATA | BE(1) | RGB(4) | BK(0)) // font12 background enabled, color red, no blink
#define FONT13 (DATA | BE(1) | RGB(4) | BK(0)) // font13 background enabled, color red, no blink
#define FONT14 (DATA | BE(1) | RGB(4) | BK(0)) // font14 background enabled, color red, no blink
#define FONT15 (DATA | BE(1) | RGB(4) | BK(0)) // font15 background enabled, color red, no blink
#define FONT16 (DATA | BE(1) | RGB(4) | BK(0)) // font16 background enabled, color red, no blink
#define FONT17 (DATA | BE(1) | RGB(4) | BK(0)) // font17 background enabled, color red, no blink
#define FONT18 (DATA | BE(1) | RGB(4) | BK(0)) // font18 background enabled, color red, no blink
#define FONT19 (DATA | BE(1) | RGB(4) | BK(0)) // font19 background enabled, color red, no blink
#define FONT20 (DATA | BE(1) | RGB(4) | BK(0)) // font20 background enabled, color red, no blink
#define FONT21 (DATA | BE(1) | RGB(4) | BK(0)) // font21 background enabled, color red, no blink

/* -- Types ---------------------------------------------------------------- */


/* -- Modul Global Function Prototypes ------------------------------------- */

static void main_got_char(void);
static void main_hscroll_redraw(void);
static void adjust_buffer_fv(uint8_t buffer_pos_u8);
static void exec_cmd(void);
static void exec_esc(void);
static void exec_display_char(uint8_t byte_u8);
static void exec_5c_cmd(void);
static uint8_t byte2font(uint8_t byte_u8);
static void move_cursor_home(void);
static void cursor_to_start_of_line(void);
static void move_cursor_right(uint8_t cursor_steps_u8);
static void move_cursor_left(uint8_t cursor_steps_u8);
static void move_cursor_down(uint8_t cursor_steps_u8);
static void move_cursor_up(uint8_t cursor_steps_u8);
static void clear_row(uint8_t row_u8);
static void vscroll_normal(void);
static void vscroll_reverse(void);
static void test_font(uint8_t byte_u8);
static void cache_char(uint8_t char_u8);
static void clear_cached_display(void);

/* -- Module Global Variables ---------------------------------------------- */

static uint8_t  x_pos;                         //1 byte, current x char position
static uint8_t  y_pos;                         //1 byte, current y char position
static uint8_t  font;                          //1 byte, current font to use
static uint8_t  cmd_len;                       //1 byte, # of chars revc
static uint8_t  cmd_buf[100];                  //30 byte command buffer
static uint8_t  hscroll_en[8];                 //8 byte
static uint8_t  vscroll_offset;                //1 byte, current y scroll offset

static uint8_t  hscroll_speed;                 //1 byte
static uint8_t  hscroll_t0_count;              //1 byte
static uint8_t  hscroll_fill_row;              //1 byte
static uint8_t  hscroll_font0[ROWS_DU8];       //8 byte
static uint8_t  hscroll_end0[ROWS_DU8];        //8 byte
static uint8_t  hscroll_pos0[ROWS_DU8];        //8 byte
static uint8_t  check_hscroll_en[ROWS_DU8];   //8 byte

static uint8_t  wrap_mode;
static uint8_t  vscroll_mode;
static uint8_t  ignore_input_mode;
static uint8_t  update_mode;
static uint8_t  hscroll_fill_en;
static uint8_t  hscroll_redraw;
static uint8_t  turn_lcd_on;
static uint8_t  handled_mu8 = FALSE;
uint8_t  runtime_max_mu8 = 0;

//external data memory
static uint8_t  hscroll_buffer[ROWS_DU8][256];       //8x256 byte bufferx for horizontal scrolling rows

//this allows 22 fonts to be stored


// this table maps an ASCII character to the character set of ST5730
static const uint8_t LookUpTable_cau8[] PROGMEM  = { 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B,    // 0x00 - 0x07
                                                     0x0B, 0x0B, 0x65, 0x65, 0x0B, 0x0B, 0x0B, 0x0B,    // 0x08 - 0x0F
                                                     0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B,    // 0x10 - 0x17
                                                     0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B,    // 0x18 - 0x1F
                                                     0x0B, 0x7D, 0x29, 0x5F, 0x1E, 0x39, 0x79, 0x29,    // 0x20 - 0x27
                                                     0x29, 0x29, 0x5F, 0x5E, 0x27, 0x0A, 0x27, 0x28,    // 0x28 - 0x2F
                                                     0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,    // 0x30 - 0x37
                                                     0x08, 0x09, 0x26, 0x27, 0x62, 0x78, 0x61, 0x70,    // 0x38 - 0x3F
                                                     0x6C, 0x0C, 0x0D, 0x0E, 0x0F, 0x10, 0x11, 0x12,    // 0x40 - 0x47
                                                     0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1A,    // 0x48 - 0x4F
                                                     0x1B, 0x1C, 0x1D, 0x1E, 0x1F, 0x20, 0x21, 0x22,    // 0x50 - 0x57
                                                     0x23, 0x24, 0x25, 0x29, 0x28, 0x29, 0x7C, 0x72,    // 0x58 - 0x5F
                                                     0x29, 0x2A, 0x2B, 0x2C, 0x2D, 0x2E, 0x2F, 0x30,    // 0x60 - 0x67
                                                     0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38,    // 0x68 - 0x6F
                                                     0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F, 0x40,    // 0x70 - 0x77
                                                     0x41, 0x42, 0x43, 0x29, 0x14, 0x29, 0x64, 0x63,    // 0x78 - 0x7F
                                                     0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B,    // 0x80 - 0x87
                                                     0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B,    // 0x88 - 0x8F
                                                     0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B,    // 0x90 - 0x97
                                                     0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B,    // 0x98 - 0x9F
                                                     0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B,    // 0xA0 - 0xA7
                                                     0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B,    // 0xA8 - 0xAF
                                                     0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B,    // 0xB0 - 0xB7 
                                                     0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B,    // 0xB8 - 0xBF
                                                     0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B,    // 0xC0 - 0xC7
                                                     0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B,    // 0xC8 - 0xCF
                                                     0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B,    // 0xD0 - 0xD7
                                                     0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B,    // 0xD8 - 0xDF
                                                     0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B,    // 0xE0 - 0xE7
                                                     0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B,    // 0xE8 - 0xEF
                                                     0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B,    // 0xF0 - 0xF7
                                                     0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B, 0x0B };  // 0xF8 - 0xFF

// the font table has the format for the STV5730
//static uint16_t font_cau16[23];

static const uint16_t font_cau16[] PROGMEM = { FONT0, 
                                               FONT1, 
                                               FONT2, 
                                               FONT3, 
                                               FONT4, 
                                               FONT5, 
                                               FONT6, 
                                               FONT7, 
                                               FONT8, 
                                               FONT9, 
                                               FONT10,
                                               FONT11,
                                               FONT12,
                                               FONT13,
                                               FONT14,
                                               FONT15,
                                               FONT16,
                                               FONT17,
                                               FONT18,
                                               FONT19,
                                               FONT20,
                                               FONT21 };

static const uint16_t Device_Select_cau16[24][3] PROGMEM  = { {0x18 + FONT1, 0x18 + FONT0, 0x18 + FONT0},     // 'M'
                                                              {0x1B + FONT1, 0x1B + FONT0, 0x1B + FONT0},     // 'P
                                                              {0x03 + FONT1, 0x03 + FONT0, 0x03 + FONT0},     // '3'
                                                              {0x0B + FONT0, 0x0B + FONT0, 0x0B + FONT0},     // ' '                                            
                                                              {0x0B + FONT0, 0x0B + FONT0, 0x0B + FONT0},     // '                                              
                                                              {0x0B + FONT0, 0x0B + FONT0, 0x0B + FONT0},     // ' '                                            
                                                              {0x0B + FONT0, 0x0B + FONT0, 0x0B + FONT0},     // ' '                                            
                                                              {0x0B + FONT0, 0x0B + FONT0, 0x0B + FONT0},     // '                                              
                                                              {0x0B + FONT0, 0x0B + FONT0, 0x0B + FONT0},     // ' '                                            
                                                              {0x0B + FONT0, 0x0B + FONT0, 0x0B + FONT0},     // ' '                                            
                                                              {0x0E + FONT0, 0x0E + FONT1, 0x0E + FONT0},     // 'C'
                                                              {0x0F + FONT0, 0x0F + FONT1, 0x0F + FONT0},     // 'D'
                                                              {0x0E + FONT0, 0x0E + FONT1, 0x0E + FONT0},     // 'C'
                                                              {0x0B + FONT0, 0x0B + FONT0, 0x0B + FONT0},     // '                                             
                                                              {0x0B + FONT0, 0x0B + FONT0, 0x0B + FONT0},     // ' '                                           
                                                              {0x0B + FONT0, 0x0B + FONT0, 0x0B + FONT0},     // ' '                                           
                                                              {0x0B + FONT0, 0x0B + FONT0, 0x0B + FONT0},     // ' '                                           
                                                              {0x0B + FONT0, 0x0B + FONT0, 0x0B + FONT0},     // '                                             
                                                              {0x0B + FONT0, 0x0B + FONT0, 0x0B + FONT0},     // ' '                                           
                                                              {0x0B + FONT0, 0x0B + FONT0, 0x0B + FONT0},     // ' '                                           
                                                              {0x0B + FONT0, 0x0B + FONT0, 0x0B + FONT0},     // '                                             
                                                              {0x0C + FONT0, 0x0C + FONT0, 0x0C + FONT1},     // 'A'
                                                              {0x20 + FONT0, 0x20 + FONT0, 0x20 + FONT1},     // 'U'
                                                              {0x23 + FONT0, 0x23 + FONT0, 0x23 + FONT1} };    // 'X



static const uint16_t Active_Device_cau16[3][3] PROGMEM  = { {0x18 + FONT8, 0x0E + FONT8, 0x0C + FONT8},     // 'M' 'C' 'A'
                                                             {0x1B + FONT8, 0x0F + FONT8, 0x20 + FONT8},     // 'P  'D' 'U'
                                                             {0x03 + FONT8, 0x0E + FONT8, 0x23 + FONT8} };    // '3' 'C' 'X

// pushbutton config
//
// autorepeat delay: no autorepeat, 400 ms, 650 ms, 900 ms
// autorepeat speed: 3/sec, 5/sec, 8/sec, 12/sec
// xmit press event
// xmit depress event

/* -- Extern Global Variables ---------------------------------------------- */

uint16_t DisplayBuffer_gau16[24][8];        // holds the current display image
extern uint8_t ir_command_mu8;

extern uint16_t start_time_mu16;
extern uint16_t high_time_mu16;
extern uint16_t low_time_mu16;

/* -- Implementation Functions --------------------------------------------- */


//-----------------------------------------------------------------------------
/*!
   \brief     void Init_Display_fv(void)

    initialise higher level functionalities associated with
    the MP3 player control and display
    
   \author     Koelling
   \date       28.12.2007

   \param[in]  void
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------


void Init_Display_fv(void)
{
   uint8_t i;
   update_mode = TRUE;
   //Init_display_HW();

   move_cursor_home();
   font = 0;
   cmd_len = 0;
   wrap_mode = TRUE;
   vscroll_mode = TRUE;
   ignore_input_mode = FALSE;
   hscroll_fill_en = FALSE;
   hscroll_redraw = FALSE;
   hscroll_fill_row = 0;
   hscroll_speed = 20;           // 20 * 10ms -> 5Hz hscroll speed
   
   for(i = 0;i < ROWS_DU8;i++)
   {
      hscroll_en[i] = 0;
      hscroll_font0[i] = 0;
      hscroll_pos0[i] = 0;
      hscroll_end0[i] = 0;
   }
   hscroll_t0_count = 0;
   
   vscroll_offset = 0;

   Set_Display_Config_fv(2,0,24,8); 
   Clear_Page_STV5730_fv();
   //cache_char('1');
}
   volatile uint8_t runtime_u8;

extern uint8_t button_gu8;

void Display_fv(void) // !!! must be cyclicly called, timeslice has to be evaluated
{
   uint8_t byte_u8;
   uint16_t display_value_u16;

   runtime_u8 = TCNT0;
   //display_value_u16 = ir_command_mu8;
   //display_value_u16 = button_gu8;
   //Send_Char_fv(runtime_u8,CONTROL_PC);

   if(turn_lcd_on == FALSE)
   {
       //display_on();
   }

   /*
   has to be replaced with remote control events and and event handler
   if(get_event() == TRUE)
   {
       print_event();
   }
   */

   if(hscroll_redraw == FALSE)
   {
      cmd_len += getchr(&cmd_buf[cmd_len]);

      if(cmd_len != 0)
      {
         main_got_char();
      }
      else
      {
    /*
         display_value_u16 = start_time_mu16;
         x_pos = 0;
         y_pos = 0;
         byte_u8 = display_value_u16 / 10000;
         cache_char(byte_u8 + 0x30);
         byte_u8 = (display_value_u16 % 10000) / 1000;
         cache_char(byte_u8 + 0x30);
         byte_u8 = (display_value_u16 % 1000) / 100;
         cache_char(byte_u8 + 0x30);
         byte_u8 = (display_value_u16 % 100) / 10;
         cache_char(byte_u8 + 0x30);
         byte_u8 = display_value_u16 % 10;
         cache_char(byte_u8 + 0x30);
         start_time_mu16 = 0;
         //cache_char(' ');
         x_pos = 0;
         y_pos = 1;
         display_value_u16 = high_time_mu16;
         byte_u8 = display_value_u16 / 10000;
         cache_char(byte_u8 + 0x30);
         byte_u8 = (display_value_u16 % 10000) / 1000;
         cache_char(byte_u8 + 0x30);
         byte_u8 = (display_value_u16 % 1000) / 100;
         cache_char(byte_u8 + 0x30);
         byte_u8 = (display_value_u16 % 100) / 10;
         cache_char(byte_u8 + 0x30);
         byte_u8 = display_value_u16 % 10;
         cache_char(byte_u8 + 0x30);
         //cache_char(' ');
         x_pos = 0;
         y_pos = 2;
         display_value_u16 = low_time_mu16;
         byte_u8 = display_value_u16 / 10000;
         cache_char(byte_u8 + 0x30);
         byte_u8 = (display_value_u16 % 10000) / 1000;
         cache_char(byte_u8 + 0x30);
         byte_u8 = (display_value_u16 % 1000) / 100;
         cache_char(byte_u8 + 0x30);
         byte_u8 = (display_value_u16 % 100) / 10;
         cache_char(byte_u8 + 0x30);
         byte_u8 = display_value_u16 % 10;
         cache_char(byte_u8 + 0x30);
         //cache_char(' ');
*/
/*
         byte_u8 = pb_state_mau8[0] / 100;
         cache_char(byte_u8 + 0x30);
         byte_u8 = (pb_state_mau8[0] % 100) / 10;
         cache_char(byte_u8 + 0x30);
         byte_u8 = pb_state_mau8[0] % 10;
         cache_char(byte_u8 + 0x30);
         cache_char(' ');
         byte_u8 = pb_state_mau8[1] / 100;
         cache_char(byte_u8 + 0x30);
         byte_u8 = (pb_state_mau8[1] % 100) / 10;
         cache_char(byte_u8 + 0x30);
         byte_u8 = pb_state_mau8[1] % 10;
         cache_char(byte_u8 + 0x30);
*/
      }
   }
   else
   {
      main_hscroll_redraw();
   }
   runtime_u8 = TCNT0 - runtime_u8;
   if(runtime_u8 > runtime_max_mu8)
   {
      runtime_max_mu8 = runtime_u8;
   }
}


//-----------------------------------------------------------------------------
/*!
   \brief     void Display_Device_Select_fv(uint8_t select_u8, uint8_t on_off_u8)

    display the status line for audio source select. mark the device to select.
    switch line on and off

   \author     Koelling
   \date       20.06.2008

   \param[in]  void
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

void Display_Device_Select_fv(DEVICE_E device_e, uint8_t on_off_u8)
{
   uint16_t char_u16;
   uint8_t i;
   
   if(on_off_u8 == TRUE)
   {
      for(i = 0;i < 24;i++)
      {
         char_u16 = pgm_read_word(&Device_Select_cau16[i][device_e]);
         putc_stv5730(i,8,char_u16);
      }
   }
   setDisplayedLine(8,on_off_u8);
}


//-----------------------------------------------------------------------------
/*!
   \brief     void Display_Device_Select_fv(uint8_t select_u8)

    display the active audio device

   \author     Koelling
   \date       10.07.2008

   \param[in]  void
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

void Display_Active_Device_fv(DEVICE_E device_e)
{
   uint16_t char_u16;
   uint8_t i;
   
   for(i = 0;i < 3;i++)
   {
      char_u16 = pgm_read_word(&Active_Device_cau16[i][device_e]);
      putc_stv5730(i +21,9,char_u16);
   }
}


//-----------------------------------------------------------------------------
/*!
   \brief     void Display_Callback_fv(void)

    timer callback function, called when a privious set timing event is over

   \author     Koelling
   \date       21.06.2008

   \param[in]  void
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

void Display_Callback_fv(void)
{
}


//-----------------------------------------------------------------------------
/*!
   \brief     static void main_got_char(void)

    get characters from UART ring buffer and arange them in lineary buffer
    for esier decoding

   \author     Koelling
   \date       28.12.2007

   \param[in]  void
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

static void main_got_char(void)
{
   uint8_t index_u8;
   index_u8 = 0;

   do
   {
      while((cmd_buf[index_u8] != 0x5C) &&
            (cmd_buf[index_u8] >= 0x20) &&
            (index_u8 < cmd_len))
      {
         if(ignore_input_mode == FALSE)  //check ignore input mode
         {
            cache_char(cmd_buf[index_u8]);
         }
         index_u8++;
      }
      adjust_buffer_fv(index_u8);      
      handled_mu8 = FALSE;
      
      index_u8 = 0;   
   
      if((cmd_buf[index_u8] == 0x5C) ||
         (cmd_buf[index_u8] < 0x20))
      {
         exec_cmd();
      }
      Send_Char_fv(cmd_buf[index_u8],CONTROL_PC);
   }
   while((cmd_len > 0) && (handled_mu8 == TRUE));
   handled_mu8 = FALSE;
}

static void adjust_buffer_fv(uint8_t buffer_pos_u8)
{
   uint8_t i;
   cmd_len -= buffer_pos_u8;
   for(i = 0;i < cmd_len;i++)
   {
      cmd_buf[i] = cmd_buf[i + buffer_pos_u8];
   }
   handled_mu8 = TRUE;
}

void Redraw_Page_fv(void)
{
   Redraw_Page_STV5730_fv(vscroll_offset);
}
//-----------------------------------------------------------------------------
/*!
   \brief     static void main_hscroll_redraw(void)

    redraw a scrolling horizontal line

   \author     Koelling
   \date       28.12.2007

   \param[in]  void
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

static void main_hscroll_redraw(void)
{
   uint8_t row_counter_u8;
   uint8_t colum_counter_u8;
   uint8_t char_pos_u8;
   uint8_t char_u8;
   uint8_t end_u8;
   uint8_t temp_x_pos_u8;
   uint8_t temp_y_pos_u8;
   uint8_t temp_font_u8;
   uint8_t temp_wrap_mode_u8;
   uint8_t temp_vscroll_mode_u8;
   uint8_t temp_update_mode_u8;

   temp_wrap_mode_u8 = wrap_mode;                          // save params
   temp_vscroll_mode_u8 = vscroll_mode;
   temp_x_pos_u8 = x_pos;         
   temp_y_pos_u8 = y_pos;
   temp_font_u8 = font;
   temp_update_mode_u8 = update_mode;
   update_mode = TRUE;
         
   for(row_counter_u8 = 0; row_counter_u8 < ROWS_DU8;row_counter_u8++)
   {
   
      if(hscroll_en[row_counter_u8] == TRUE)
      {
         x_pos = 0;                                              // init params
         y_pos= row_counter_u8;
         char_pos_u8 = hscroll_pos0[row_counter_u8];             // get first char location
         font = hscroll_font0[row_counter_u8];                   // get font for this line
         end_u8 = hscroll_end0[row_counter_u8];                  // point to end
         wrap_mode = FALSE;
         vscroll_mode = FALSE;
         
         for(colum_counter_u8 = 0;colum_counter_u8 < COLUMS_DU8;colum_counter_u8++)
         {
            char_u8 = hscroll_buffer[row_counter_u8][char_pos_u8];
            cache_char(char_u8);
            char_pos_u8++;
            if(char_pos_u8 == end_u8)
            {
               char_pos_u8 = 0;
            }
         }
         Redraw_Row_STV5730_fv(row_counter_u8, vscroll_offset);
         hscroll_pos0[row_counter_u8]++;
         if(hscroll_pos0[row_counter_u8] == end_u8) hscroll_pos0[row_counter_u8] = 0;
      }
   }
   hscroll_redraw = FALSE;
   x_pos = temp_x_pos_u8;                                // restore params
   y_pos = temp_y_pos_u8;
   font = temp_font_u8;
   wrap_mode = temp_wrap_mode_u8;                          
   vscroll_mode = temp_vscroll_mode_u8;
   update_mode = temp_update_mode_u8;
}


//-----------------------------------------------------------------------------
/*!
   \brief     static void exec_cmd(void)

    Handle Control Characters

   \author     Koelling
   \date       28.12.2007

   \param[in]  void
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

static void exec_cmd(void)
{
   uint8_t character_u8;
   uint8_t cursor_steps_u8;
   // get character from UART buffer
   character_u8 = cmd_buf[0];
   switch(character_u8)
   {
   case 0x5C:
      exec_5c_cmd();
      break;    
   case 27:                      // ESC Sequence
      exec_esc();
      break;
   default:
      if(ignore_input_mode == 0)
      {
         switch(character_u8)
         {
         case 1:                 // CTRL-A: Cursor Home
            //display_on();
            move_cursor_home();
            break;
         case 8:                 // CTRL-H: Backspace
            move_cursor_left(1);
            break;
         case 9:                 // CTRL-I: Tab
            cursor_steps_u8 = TAB_WIDTH - (x_pos & TAB_WIDTH_MASK);
            move_cursor_right(cursor_steps_u8);
            break;
         case 10:               // CTRL-J: Line Feed
            move_cursor_down(1);
            break;
         case 11:               // CTRL-K: Up one line
            move_cursor_up(1);
            break;
         case 12:               // CTRL-L: Form Feed (clear display)
            //init_display();
            move_cursor_home();
            break;
         case 13:               // CTRL-M: Carriage Return
            cursor_to_start_of_line();
            break;
         case 21:               // CTRL-U: Forward
            move_cursor_right(1);
            break;
         default:               // just ignore unknown control characters
            break;
         }                      
      }
      adjust_buffer_fv(1);
      break;
   }
}


//-----------------------------------------------------------------------------
/*!
   \brief     static void exec_esc(void)

    Handle Escape Sequences
    
    0x1B 0x5B 0x41, move up one line
    0x1B 0x5B 0x42, move down one line
    0x1B 0x5B 0x43, move right one line
    0x1B 0x5B 0x44, move left one line
    0x1B 0x5B 0x48, cursor home
    0x1B 0x5B 0x30 0x3B 0x37 0x6B, begin inverse text
    0x1B 0x5B 0x30 0x6B, begin normal text
    0x1B 0x5B 0x32 0x4A, clear screen

   \author     Koelling
   \date       28.12.2007

   \param[in]  void
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------


static void exec_esc(void)             
{
   if(cmd_len >= 2)             // if len < 2, wait for more
   {
      switch(cmd_buf[1])
      {
      case 0x5B:   
         // TODO: this parsing really ought to allow numeric and ';'
         // characters and conclude that we have a complete escape
         // sequence when we get a alpha character A-Z or a-z.
         if(cmd_len >= 3)    // if len < 3, wait for more
         {
            if(ignore_input_mode == 0)
            {
               switch(cmd_buf[2])
               {
               case 0x41:        // exec_esc_5B_41
                  move_cursor_up(1);
                  break;
               case 0x42:        // exec_esc_5B_42
                  move_cursor_down(1);
                  break;
               case 0x43:        // exec_esc_5B_43
                  move_cursor_right(1);
                  break;
               case 0x44:        // exec_esc_5B_44
                  move_cursor_left(1);
                  break;
               default:          // exec_esc_5B_unknown
                  break;
               }
            }
            adjust_buffer_fv(3);  
         }
         break;
      case 0x1B:                 // ignore the 1st ESC
         adjust_buffer_fv(1);
         break;
      default:
         // cmd_buf[1] is something else
         adjust_buffer_fv(2);   
         break;
      }
   }
}


//-----------------------------------------------------------------------------
/*!
   \brief     exec_display_char(byte_u8)

    display one character on the display

   \author     Koelling
   \date       28.12.2007

   \param[in]  byte_u8 -> character to display
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

static void exec_display_char(uint8_t byte_u8)
{
   cache_char(byte_u8);
   adjust_buffer_fv(2);
}


//-----------------------------------------------------------------------------
/*!
   \brief     exec_5c_cmd

    Handle Custom Display Commands

   \author     Koelling
   \date       28.12.2007

   \param[in]  void
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

static void exec_5c_cmd(void)
{
   uint8_t byte_u8;
   uint8_t font_u8;
   uint8_t index_u8;
   uint8_t counter_u8;

   uint8_t bank_pin1; // dummy
   uint8_t bank_pin2; // dummy
   uint8_t led_pin; // dummy
   //adjust_buffer_fv(1);
   
   if(cmd_len >= 2)                              // if len < 2, wait for more
   {
      switch(cmd_buf[1])
      {
      case 0x5C:                              // cmd 0x5C 0x5C, display '\'
         exec_display_char(0x5C);
         break;             
      case 0x5F:                              // cmd 0x5C 0x5F, display byte 127
         exec_display_char(127);
         break;             
      case 0x5D:                              // begin ignoring input bytes (but not commands)
         ignore_input_mode = 1;
         adjust_buffer_fv(2);
         break;
      case 0x5B:                              // don't ignore input bytes)
         ignore_input_mode = 0;
         adjust_buffer_fv(2);
         break;
      case 0x3f:
         if(cmd_len >= 3)
         {
            switch(cmd_buf[2])
            {
            case 0x30:
               byte_u8 = runtime_max_mu8 / 100;
               cache_char(byte_u8 + 0x30);
               byte_u8 = (runtime_max_mu8 % 100) / 10;
               cache_char(byte_u8 + 0x30);
               byte_u8 = (runtime_max_mu8 % 100) % 10;
               cache_char(byte_u8 + 0x30);
               break;
            case 0x31:
               break;
            default:
               break;
            }
            adjust_buffer_fv(3);
         }
         break;
      case 0x40:                              // cmd 0x5C 0x40 <font+32> <opcode>, clear display or show font summary
         if(cmd_len >= 4)
         {
            byte_u8 = cmd_buf[2];
            if((byte_u8 > 31) && (byte_u8 < 48))
            {
               switch(cmd_buf[3])
               {
               case 0x30:
                  clear_cached_display();
                  Clear_Page_STV5730_fv();
                  vscroll_offset = 0;
                  break;
               case 0x31:
                  test_font(0);
                  //msg_f0
                  //font_preview
                  break;
               case 0x32:
                  test_font(128);
                  //msg_f128
                  //font_preview
                  break;
               default:
                  break;
               }
            }
            move_cursor_home();
            adjust_buffer_fv(4);
         }
         break;
      case 0x41:                              // cmd 0x5C 0x41 <mode>, set modes (wrap and vscroll)
         if(cmd_len >= 3)
         {
            switch(cmd_buf[2])
            {
            case 0x30:
               wrap_mode = FALSE;
               vscroll_mode = FALSE;
               break;
            case 0x31:
               wrap_mode = TRUE;
               vscroll_mode = FALSE;
               break;
            case 0x32:
               wrap_mode = FALSE;
               vscroll_mode = TRUE;
               break;
            case 0x33:
               wrap_mode = TRUE;
               vscroll_mode = TRUE;
               break;
            }
            adjust_buffer_fv(3);
         }
         break;
      case 0x42:                              // cmd 0x5C 0x42 <column+32> <row+32>, set cursor position
         if(cmd_len >= 4)
         {
            x_pos = (cmd_buf[2] + 224); 
            if(x_pos > (COLUMS_DU8 -1))
            {
               x_pos = COLUMS_DU8 -1;
            }
            y_pos = (cmd_buf[3] + 224);
            if(y_pos > (ROWS_DU8 -1))
            {
               y_pos = ROWS_DU8 -1;
            }
            adjust_buffer_fv(4);
         }
         break;
      case 0x43:                              // cmd 0x5C 0x43 <font+32>, set font
         if(cmd_len >= 3)
         {
            byte_u8 = cmd_buf[2];
            if((byte_u8 > 31) && (byte_u8 < 48))
            {
               font = byte_u8 - 32;
            }
            adjust_buffer_fv(3);
         }
         break;
      case 0x44:                              // cmd 0x5C 0x44 <char (2 bytes hex)> <font+32> <bitmap (8 bytes)>
         adjust_buffer_fv(2);                 // we don't support customer defined characters
         break;                                
      case 0x45:
         //cout(0xFF);
         //cout(0xC3);
         //printf(msg_version);
         //cout(1);
         cmd_len = 0;
         break;
      case 0x46:
         if(cmd_len >= 3)
         {
            byte_u8 = cmd_buf[2];
            if(byte_u8 > 31)
            {
               if(byte_u8 > 63)
               {
                  if(byte_u8 > 95)
                  {
                     if(byte_u8 > 111)
                     {
                        if(byte_u8 > 127)
                        {
                           move_cursor_up(byte_u8 - 128);
                        }
                        else
                        {
                           move_cursor_up(byte_u8 - 112);
                        }
                     }
                     else
                     {
                        move_cursor_down(byte_u8 - 96);
                     }
                  }
                  else
                  {   
                     move_cursor_right(byte_u8 - 64);
                  }
               }
               else
               {
                  move_cursor_left(byte_u8 - 32);  
               }
            }
            adjust_buffer_fv(3);
         }
         break;
      case 0x47:                              // cmd 0x5C 0x47 <button+48> <config+32>, set button config
         if(cmd_len >= 4)
         {
            byte_u8 = cmd_buf[2] - 48;
            if(byte_u8 < 12)
            {
               //pb_config[byte_u8] = cmd_buf[3];
            }
            adjust_buffer_fv(4);
         }
         break;
      case 0x48:                              // cmd 0x5C 0x48 <button> <string, 12 bytes>
         // button 48 - 59 = string for button down   ASCII: 0 1 2 3 4 5 6 7 8 9 : ;
         // button 64 - 75 = string for button up     ASCII: @ A B C D E F G H I J K 
         // button 80 - 91 = string for button repeat ASCII: P Q R S T U V W X Y Z [
         if(cmd_len >= 15)
         {
            byte_u8 = cmd_buf[2] - 48;
               
            if((byte_u8 & 0x0F)  < 12)
            {
               index_u8 = 1;
               if(byte_u8 > 15) 
               {
                  byte_u8 -= 16;
                  index_u8++;
                  if(byte_u8 > 15) 
                  {
                     byte_u8 -= 16;
                     index_u8++;
                  }
               }
               cmd_buf[14] = ' '; // ensures terminating 0 in the following loop
               counter_u8 = 14;
               
               while((counter_u8 > 2) && (cmd_buf[counter_u8] == ' '))
               {
                  cmd_buf[counter_u8] = 0;
                  counter_u8--;
               }
               PB_String_Write_fv(byte_u8 & 0x0F, index_u8, &cmd_buf[3]);
            }
            adjust_buffer_fv(15);
         }
         break;
      case 0x49:                              // cmd 0x5c 0x49 <param>
         // redraw all (param = '8') or a row ('0' <= param <= '7')
         // of the screen from the cache
         // turn display updates on '9' or off ':'
         if(cmd_len >= 3)
         {
            switch(cmd_buf[2])
            {
            case 0x38:
               Redraw_Page_STV5730_fv(vscroll_offset);
               break;
            case 0x39:
               update_mode = 1;
               break;
            case 0x3A:
               update_mode = 0;
               break;
            default:
               Redraw_Row_STV5730_fv((cmd_buf[2] & 0x07), vscroll_offset);
               break; 
            }
            adjust_buffer_fv(3);
         }
         break;
      case 0x4A:                              // cmd 0x5c 0x4a <'0'|'1'|'2'> <'0'|'1'> write
                                              // cmd 0x5c 0x4a <'4'|'5'|'6'> <'0'>     read
         if(cmd_len >= 4)
         {
            switch(cmd_buf[2] & 0x07)
            {
            case 0:
               if((cmd_buf[3] & 0x01) == 1) led_pin = 0;
               else led_pin = 1;
               break;
            case 1:
               if((cmd_buf[3] & 0x01) == 1) bank_pin1 = 0;
               else bank_pin1 = 1;
               break;
            case 2:
               if((cmd_buf[3] & 0x01) == 1) bank_pin2 = 0;
               else bank_pin2 = 1;
               break;
            case 4:
               // printf(mesg_io_read);
               //if(led_pin == 1) cout(0);
               //else cout(1);
               break;
            case 5:
               // printf(mesg_io_read);
               //if(bank_pin1 == 1) cout(0);
               //else cout(1);
               break;
            case 6:
               // printf(mesg_io_read);
               //if(bank_pin2 == 1) cout(0);
               //else cout(1);
               break;
            default: break;
            }
            adjust_buffer_fv(4);
         }
         break;
      case 0x4B:                              // cmd 0x5c 'K' control horizontal scrolling display element
         if(cmd_len >= 5)
         {
            switch(cmd_buf[2] & 0x07)
            {
            case 0x30:                        // 0x5c 'K' '0' <'0' - '7'> '0' - set speed
               if((cmd_buf[3] & 0xF8) == 0x30)
               {
                  hscroll_speed = (cmd_buf[3] +1) * 2;
                  hscroll_t0_count = 0;
               }
               break;
            case 0x31:                        // 0x5c 'K' '1' <row+32> <font+32> - set font for row
               byte_u8 = cmd_buf[3];
               font_u8 = cmd_buf[4];
               if( ( (byte_u8 & 0xF8) == 0x20) &&
                   ( (font_u8 & 0xF8) == 0x20) )
               {
                  hscroll_font0[byte_u8 & 0x07] = font_u8 & 0x07;
               }
               break;
            case 0x32:                        // 0x5c 'K' '2' <row+32> <'0'|'1'> - disable/enable row
               byte_u8 = cmd_buf[3];
               if((byte_u8 & 0xF8) == 0x20)
               {
                  if((cmd_buf[4] & 0xFE) == 0x30) // get setting
                  {
                     hscroll_fill_en = 0;         // disable fill
                     if(cmd_buf[4] == 0x31)
                     {
                        if(hscroll_end0[byte_u8 & 0x07] != 0)
                        {
                           hscroll_en[(byte_u8 & 0x07)] = TRUE;
                        }
                        else if(cmd_buf[4] == 0x30)
                        {
                           hscroll_en[(byte_u8 & 0x07)] = FALSE;
                        }
                     }
                  }
               }
               break;
            case 0x33:                        // 0x5c 'K' '3' <row+32> <'0'|'1'> - enable/disable buffer fill for row
               byte_u8 = cmd_buf[3];
               if((byte_u8 & 0xF8) == 0x20)
               {  
                  hscroll_fill_row = byte_u8 & 0x07;
                  if(cmd_buf[4] == 0x31)
                  {
                     hscroll_fill_en = 1;
                     hscroll_end0[hscroll_fill_row] = 0;
                     hscroll_pos0[hscroll_fill_row] = 0;
                  }
                  else
                  {
                     hscroll_fill_en = 0;
                  }
               }
               break;
            }
            adjust_buffer_fv(5);
         }
         break;
      case 0x4C:                               // cmd 0x5c 0x4c <param>
                                                // clear a row ('0' <= param <= '7') of the display
         if(cmd_len >= 3)
         {
            if((cmd_buf[2] & 0xF8) == 0x20)
            {
               clear_row(cmd_buf[2] & 0x07);
            }
            adjust_buffer_fv(3);
         }
         break;
      default:                                
         if((cmd_buf[1] & 0xE0) == 0x20)      // cmd 0x5C <0x20-0x3F>, display bytes 0 to 31
         {
            byte_u8 = cmd_buf[1] + 224;       
            exec_display_char(byte_u8);
         }
         else if((cmd_buf[1] & 0xE0) == 0x60) // cmd 0x5C <0x60-0x7F>, display bytes 128 to 159
         {
            byte_u8 = cmd_buf[1] + 32;       
            exec_display_char(byte_u8);
         }
         else
         {
            //phex(cmd_buf[1]);
            //cout('*');
            adjust_buffer_fv(2);
         }
         break;
      }
   }
}


//***********************************************************************
//**                                                                   **
//**                         Cursor Movement                           **
//**                                                                   **
//***********************************************************************


//-----------------------------------------------------------------------------
/*!
   \brief     move_cursor_home

    set cursor to first position in first line

   \author     Koelling
   \date       08.01.2008

   \param[in]  void
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

static void move_cursor_home(void)
{
   x_pos = 0;
   y_pos = 0;
}


//-----------------------------------------------------------------------------
/*!
   \brief     cursor_to_start_of_line

    set cursor to first position in current line

   \author     Koelling
   \date       08.01.2008

   \param[in]  void
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

static void cursor_to_start_of_line(void)
{
   x_pos = 0;
}


//-----------------------------------------------------------------------------
/*!
   \brief     move_cursor_right(cursor_steps_u8)

    move the cursor right by cursor_steps_u8

   \author     Koelling
   \date       08.01.2008

   \param[in]  cursor_steps_u8 -> number of steps to move cursor
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

static void move_cursor_right(uint8_t cursor_steps_u8)
{
   x_pos += cursor_steps_u8; 
   if(x_pos > (COLUMS_DU8 - 1))
   {
      if(wrap_mode == FALSE) 
      {
         x_pos = COLUMS_DU8 - 1;
      }
      else
      {
         move_cursor_down(1);
         cursor_steps_u8 = x_pos - COLUMS_DU8;
         x_pos = 0;
         if(cursor_steps_u8 > 0)
         {
            move_cursor_right(cursor_steps_u8);
         }
      }
   }
}


//-----------------------------------------------------------------------------
/*!
   \brief     move_cursor_right(cursor_steps_u8)

    move the cursor left by cursor_steps_u8

   \author     Koelling
   \date       08.01.2008

   \param[in]  cursor_steps_u8 -> number of steps to move cursor
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

static void move_cursor_left(uint8_t cursor_steps_u8)
{
   if(x_pos >= cursor_steps_u8)
   {
      x_pos -= cursor_steps_u8;  // easy case, no wrap around
   }
   else
   {
      if(wrap_mode == FALSE) 
      {
         x_pos = 0;             // left edge when wrap mode off
      }
      else
      {
         move_cursor_up(1);
         cursor_steps_u8 -= x_pos;
         x_pos = COLUMS_DU8 - 1;
         move_cursor_left(cursor_steps_u8);
      }
   }
}


//-----------------------------------------------------------------------------
/*!
   \brief     move_cursor_down(cursor_steps_u8)

    move the cursor down by cursor_steps_u8

   \author     Koelling
   \date       08.01.2008

   \param[in]  cursor_steps_u8 -> number of steps to move cursor
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

static void move_cursor_down(uint8_t cursor_steps_u8)
{
   uint8_t step_u8;
   
   y_pos += cursor_steps_u8;
   if(y_pos > (ROWS_DU8 - 1))
   {
      if(vscroll_mode == TRUE)
      {
         step_u8 = y_pos - (ROWS_DU8 - 1);
         y_pos = ROWS_DU8 - 1;
         do
         {
            vscroll_normal();
            step_u8--;
         }
         while(step_u8 > 0);

         Redraw_Page_STV5730_fv(vscroll_offset);

      } 
      else if(wrap_mode == TRUE)
      {
         step_u8 = y_pos - ROWS_DU8;
         y_pos = 0;
         move_cursor_down(step_u8);
      }
      else
      {
         y_pos = ROWS_DU8 - 1;         // bottom edge if no wrap or vertical scroll
      }
   }
}

//-----------------------------------------------------------------------------
/*!
   \brief     move_cursor_up(cursor_steps_u8)

    move the cursor up by cursor_steps_u8

   \author     Koelling
   \date       11.01.2008

   \param[in]  cursor_steps_u8 -> number of steps to move cursor
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

static void move_cursor_up(uint8_t cursor_steps_u8)
{
   uint8_t step_u8;
   
   if(cursor_steps_u8 > y_pos)
   {
      if(vscroll_mode == TRUE)
      {
         step_u8 = cursor_steps_u8 - y_pos;
         y_pos = 0;
         do
         {
            vscroll_reverse();
            step_u8--;
         }
         while(step_u8 > 0);
         Redraw_Page_STV5730_fv(vscroll_offset);
      } 
      else if(wrap_mode == TRUE)
      {
         step_u8 = cursor_steps_u8 - y_pos - 1;
         y_pos = 7;
         move_cursor_up(step_u8);
      }
      else
      {
         y_pos = 0;
      }
   }
   else
   {
      y_pos -= cursor_steps_u8;        // easy case, didn't hit the top
   }
}

//-----------------------------------------------------------------------------
/*!
   \brief     clear_row(row_u8)

    clear the row specified by row_u8

   \author     Koelling
   \date       19.01.2008

   \param[in]  row_u8 -> row to clear
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

static void clear_row(uint8_t row_u8)
{
   uint8_t i;
   uint8_t row_buffer_u8;
   uint16_t char_u16;
   
   char_u16 = (uint16_t)(pgm_read_byte(&LookUpTable_cau8[' '])) + pgm_read_word(&font_cau16[font]);
   char_u16 = LookUpTable_cau8[' '] + font_cau16[font];
   row_buffer_u8 = (vscroll_offset + row_u8) & (ROWS_DU8  - 1);
   for(i = 0;i < COLUMS_DU8;i++)
   {
      DisplayBuffer_gau16[i][row_buffer_u8] = char_u16;
   }
   if(update_mode == TRUE) // Achtung bei Verwendung aus v_scroll heraus!!!
   {
      Clear_Row_STV5730_fv(row_u8);
   }
}


//-----------------------------------------------------------------------------
/*!
   \brief     vscroll_normal(void)

    scroll one line up

   \author     Koelling
   \date       19.01.2008

   \param[in]  void
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

static void vscroll_normal(void)
{
   uint8_t update_mode_temp_u8;
   
   vscroll_offset++;
   vscroll_offset &= (ROWS_DU8 - 1);
   update_mode_temp_u8 = update_mode;
   update_mode = FALSE;
   clear_row(y_pos);
   update_mode = update_mode_temp_u8;
}


//-----------------------------------------------------------------------------
/*!
   \brief     vscroll_reverse(void)

    scroll one line down

   \author     Koelling
   \date       19.01.2008

   \param[in]  void
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

static void vscroll_reverse(void)
{
   uint8_t update_mode_temp_u8;
   
   vscroll_offset--;
   vscroll_offset &= (ROWS_DU8 - 1);
   update_mode_temp_u8 = update_mode;
   update_mode = FALSE;
   clear_row(y_pos);
   update_mode = update_mode_temp_u8;
}


//***********************************************************************
//**                                                                   **
//**                   Fonts and Font Manipulation                     **
//**                                                                   **
//***********************************************************************


static void test_font(uint8_t byte_u8)
{
}

//**************************************************************************
//**                                                                      **
//**         Character Caching Routines                                    **
//**                                                                      **
//**************************************************************************

static void cache_char(uint8_t char_u8)
{
   uint8_t row_u8;
   uint16_t char_u16;                   // char in STV5730 format
   uint8_t hscroll_end_u8;

   if(hscroll_fill_en == TRUE)
   {
      hscroll_end_u8 = hscroll_end0[hscroll_fill_row];
      hscroll_buffer[hscroll_fill_row][hscroll_end_u8] = char_u8;
      hscroll_end0[hscroll_fill_row]++;
   }
   else
   {
      row_u8 = (y_pos + vscroll_offset) & (ROWS_DU8 - 1);
      if(check_hscroll_en[row_u8] == FALSE) // ignore updates to scrolling rows
      {
         // convert character from ASCII to STV5730 character set and add the charater font
         char_u16 = (uint16_t)(pgm_read_byte(&LookUpTable_cau8[char_u8])) + pgm_read_word(&font_cau16[font]);
         DisplayBuffer_gau16[x_pos][row_u8] = char_u16;
         if(update_mode == TRUE)
         {
            putc_stv5730(x_pos,y_pos,char_u16);
         }  
         move_cursor_right(1);
      }
    }
}


static void clear_cached_display(void)
{
   uint8_t i;
   uint8_t j;
   uint16_t char_u16;
   
   char_u16 = (uint16_t)(pgm_read_byte(&LookUpTable_cau8[' '])) + pgm_read_word(&font_cau16[font]);

   for(i = 0;i < ROWS_DU8;i++)
   {
      for(j = 0; j < COLUMS_DU8;j++)
      {
         DisplayBuffer_gau16[j][i] = char_u16;
      }
   }
}

// eof
