/*****************************************************************************/
/*!
\file      ir.c
\brief     remote control  and event handling 
                         

\implementation
  project       Multimedia Gateway
  version       0.00
  copyright     taken from RoboterNETZ 
  
  generated     Matthias Koelling 03.03.2008
  review
  released

  compiler      AVR GCC
  target        ATmega128-16AU
\endimplementation

\history  
  Rev     Date         Author      Commentary
  0.00    03.03.2008   Koelling    generated
\endhistory  
*/
/*****************************************************************************/

/* -- Includes ------------------------------------------------------------- */

#include <avr/io.h>
#include <stdlib.h>
#include <string.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "main.h"
#include "hardware.h"
#include "vwcdc.h"
#include "com.h"
#include "tda4886.h"
#include "stv5730.h"
#include "device.h"
#include "display.h"
#include "ir.h"

/* -- Configuration Parameters --------------------------------------------- */

/* -- Makros --------------------------------------------------------------- */

/* -- Defines -------------------------------------------------------------- */ 

#define DISP_OFF            13
#define ATT                 20
#define SOURCE              70
#define SEL                 17
#define MODE                71
#define UP                  51
#define LEFT                53
#define RIGHT               52
#define DOWN                50
#define DISPL               40
#define SCRL                35
#define _1                   0
#define _2                   1
#define _3                   2
#define _4                   3
#define _5                   4
#define _6                   5
#define VOL_UP              18
#define VOL_DOWN            19
#define REPEAT              68
#define RELEASED            69

#define SONY_START           0
#define SONY_START_BIT       1
#define SONY_DATA            2
#define SONY_ADDR            3
#define SONY_DATA_LEN        7
#define SONY_ADDR_LEN       15
#define SONY_STARTBIT       47
#define SONY_STARTBIT_MIN   41
#define SONY_STARTBIT_MAX   53
#define SONY_HIGH_LEN       25
#define SONY_ADDRESS       194
#define SONY_BUTTON         19  // butons on ir control
#define DISPLAY_BUTTON      12  // button for mp3 control

#define IR_INPUT     INT7_vect

/* -- Types ---------------------------------------------------------------- */

typedef struct
{
   void(*PB_Dn_Func_pfv)(uint8_t button_u8); // function pointer for pressed button
   void(*PB_Up_Func_pfv)(uint8_t button_u8); // function pointer for released button
   void(*PB_Ar_Func_pfv)(uint8_t button_u8); // function pointer for auto repeat button
}PB_FUNC_TAB_t;

/* -- Modul Global Function Prototypes ------------------------------------- */

static void MP3_Dn_Control_fv(uint8_t button_u8);
static void MP3_Up_Control_fv(uint8_t button_u8);
static void MP3_Ar_Control_fv(uint8_t button_u8);
static void Display_Dn_Control_fv(uint8_t button_u8);
static void Display_Up_Control_fv(uint8_t button_u8);
static void Display_Ar_Control_fv(uint8_t button_u8);
static void Source_Dn_Control_fv(uint8_t button_u8);
static void Source_Up_Control_fv(uint8_t button_u8);
static void Source_Ar_Control_fv(uint8_t button_u8);
static void Display_Button_Strings_fv(void);

static void _2_fv(uint8_t button_u8);
static void _5_fv(uint8_t button_u8);

/* -- Module Global Variables ---------------------------------------------- */

uint8_t ir_command_mu8 = 0xFF;             // default value -> no action
static uint8_t pb_config_mau8[SONY_BUTTON] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
static uint8_t ir_counter_mu8;
// control strings for each of 12 push buttons for mp3 control 
static pb_strings_t pb_strings_at[DISPLAY_BUTTON]; 
static const uint8_t ar_threshold_cau8[] PROGMEM = {20, 15, 11, 7};
static const uint8_t ar_speed_cau8[] PROGMEM = {7, 6, 5, 4, 3, 2, 1, 0};
static const uint8_t button_config_cau8[] PROGMEM = { VOL_UP, 
                                                      VOL_DOWN,
                                                      _6,
                                                      _3,
                                                      _4,
                                                      DISPL,
                                                      _1,
                                                      UP,
                                                      LEFT,
                                                      RIGHT,
                                                      DOWN,
                                                      SCRL,
                                                      DISP_OFF,
                                                      ATT,
                                                      SOURCE,
                                                      SEL,
                                                      MODE,
                                                      _2,
                                                      _5 };

static const PB_FUNC_TAB_t PB_Func_Tab_t[SONY_BUTTON] PROGMEM =
{
   { MP3_Dn_Control_fv , MP3_Up_Control_fv , MP3_Ar_Control_fv },  // VOL_UP 
   { MP3_Dn_Control_fv , MP3_Up_Control_fv , MP3_Ar_Control_fv },  // VOL_DOWN
   { MP3_Dn_Control_fv , MP3_Up_Control_fv , MP3_Ar_Control_fv },  // _6
   { MP3_Dn_Control_fv , MP3_Up_Control_fv , MP3_Ar_Control_fv },  // _3
   { MP3_Dn_Control_fv , MP3_Up_Control_fv , MP3_Ar_Control_fv },  // _4
   { MP3_Dn_Control_fv , MP3_Up_Control_fv , MP3_Ar_Control_fv },  // DISPL
   { MP3_Dn_Control_fv , MP3_Up_Control_fv , MP3_Ar_Control_fv },  // _1
   { MP3_Dn_Control_fv , MP3_Up_Control_fv , MP3_Ar_Control_fv },  // UP
   { MP3_Dn_Control_fv , MP3_Up_Control_fv , MP3_Ar_Control_fv },  // LEFT
   { MP3_Dn_Control_fv , MP3_Up_Control_fv , MP3_Ar_Control_fv },  // RIGHT
   { MP3_Dn_Control_fv , MP3_Up_Control_fv , MP3_Ar_Control_fv },  // DOWN
   { MP3_Dn_Control_fv , MP3_Up_Control_fv , MP3_Ar_Control_fv },  // SCRL
   { Display_Dn_Control_fv , Display_Up_Control_fv , NULL },       // DISP_OFF
   { NULL , NULL , NULL },                                         // ATT
   { Change_Device_fv , NULL , NULL },                             // SOURCE
   { Select_Device_fv , NULL , NULL },                             // SEL
//   { Display_Button_Strings_fv , NULL , NULL },                    // MODE
   { Source_Ar_Control_fv , NULL , NULL },                    // MODE
   { _2_fv , NULL , NULL },                                         // _2
   { _5_fv , NULL , NULL }                                          // _5
};
    

/* -- Extern Global Variables ---------------------------------------------- */

extern const uint8_t sNEWLINE[];
uint8_t button_gu8;

/* -- Implementation Functions --------------------------------------------- */

//-----------------------------------------------------------------------------
/*!
   \brief     Init_IR_fv

    initialization of IR decoder and event handler

   \author     Koelling
   \date       03.03.2008

   \param[in]  void
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

void Init_IR_fv(void)
{
   EICRB = 1<<ISC71; // interrupt on falling edge of INT7
   EIMSK = 1<<INT7;  // enable interrupt
}

//-----------------------------------------------------------------------------
/*!
   \brief     INTERRUPT (IR_INPUT)

    external interrupt service routine remote control decoder

   \author     Koelling
   \date       03.03.2008

   \param[in]  none
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

ISR(IR_INPUT)
{
   static uint8_t Sony_timer_hu8;
   static uint8_t Sony_timer_old_hu8 = 0;
   static uint8_t Sony_State_hu8 = SONY_START;
   static uint8_t Sony_Data_hu8;
   static uint8_t Sony_Addr_hu8;
   static uint8_t Sony_count_hu8;

   Sony_timer_hu8 = TCNT0 - Sony_timer_old_hu8;
   Sony_timer_old_hu8  = TCNT0; 
   switch(Sony_State_hu8)
   {
   case SONY_START:
      Sony_count_hu8 = 0;
      Sony_Data_hu8 = 0;
      Sony_Addr_hu8 = 0;
      Sony_State_hu8 = SONY_START_BIT;
   break;
   case SONY_START_BIT:
      if((Sony_timer_hu8 >= SONY_STARTBIT_MIN) &&
         (Sony_timer_hu8 <= SONY_STARTBIT_MAX))
      {   
         Sony_State_hu8 = SONY_DATA;
      }
      break;
   case SONY_DATA:
      if(Sony_count_hu8 >= SONY_DATA_LEN)
      {
         Sony_Addr_hu8 >>= 1;
         if(Sony_timer_hu8 >= SONY_HIGH_LEN)
         {
           Sony_Addr_hu8 |= 0x80;
         }
         if(Sony_count_hu8 >= SONY_ADDR_LEN)
         {
            Sony_timer_old_hu8 = 0;
            Sony_State_hu8 = SONY_START;
            if(Sony_Addr_hu8 == SONY_ADDRESS)
            {
               Sony_Data_hu8 >>= 1;
               // the rc sends code 68 for some buttons as autorepeat indication
               if(Sony_Data_hu8 != 68)                    
               {
                  ir_command_mu8 = Sony_Data_hu8; // the code is overtaken
               }                                  // otherwise we use last ir code  
               ir_counter_mu8 = 0x80;             // indicate that a new command was received 
            }
            else
            {
               ir_command_mu8 = 0xFF;
            }
            
         }
      }
      else
      {
         Sony_Data_hu8 >>= 1;
         if(Sony_timer_hu8 >= SONY_HIGH_LEN) 
         {
            Sony_Data_hu8 |= 0x80;
         }
      }
      Sony_count_hu8++;
      break;
   default:
      Sony_count_hu8 = 0;
      Sony_Data_hu8 = 0;
      Sony_Addr_hu8 = 0;
      Sony_State_hu8 = SONY_START;
      Sony_timer_old_hu8 = 0;
      break;
  
   }
   //PORTC &= ~_BV(MP3_ENABLE);     
}

//-----------------------------------------------------------------------------
/*!
   \brief     void IR_Command_50ms_fv(void)

    process ir commands and put events in queue

   \author     Matthias Koelling
   \date       11.03.2008    

   \param[in]  none
   \param[out] none
   \return		void
*/
//-----------------------------------------------------------------------------

void IR_Command_50ms_fv(void)
{
	uint8_t button_u8;
   uint8_t ar_thresh_u8;
   uint8_t ar_speed_u8;
   uint8_t button_config_u8;

   void(*func_pfv)(uint8_t button_u8);      // function pointer button action
   
   static uint8_t pb_state_hau8[SONY_BUTTON];
   static uint8_t ir_command_hu8 = 0xFF;    // initialize with invalid command
  

   if(ir_counter_mu8 == 0x80)               // was a new command received ?
   {
      ir_command_hu8 = ir_command_mu8;      // then take this command
   }
   if(ir_counter_mu8 & 0x80)                // was there a new command at least 2 calls before ?
   {
      ir_counter_mu8++;                     // increment it for timeout detection 
      if(ir_counter_mu8 > 0x82)             // have we reached a timeout     
      {
         ir_command_hu8 = 0xFF;             // 0xFF is not valid command -> last action is truncated 
         ir_counter_mu8 = 0;                // reset the timeout counter
      }
   }
   for(button_u8 = 0;button_u8 < SONY_BUTTON;button_u8++)
   {
      ar_thresh_u8 = pb_config_mau8[button_u8] & 0x03;
      ar_speed_u8 = (pb_config_mau8[button_u8] >> 2) & 0x07;
      button_config_u8 = pgm_read_byte(&button_config_cau8[button_u8]);
      
      if(button_config_u8 == ir_command_hu8)                   // the button is being pressed right now!
      {
         if(pb_state_hau8[button_u8] == 2)                     // first hit
         {
            func_pfv = (void*)pgm_read_word(&PB_Func_Tab_t[button_u8].PB_Dn_Func_pfv);
            if(func_pfv != NULL)
            {
               func_pfv(button_u8);
            }
         }
         else                                                       // detect auto repeat
         {
            if(pb_state_hau8[button_u8] > pgm_read_byte(&ar_threshold_cau8[ar_thresh_u8])) // did we hit the auto-repeat threshold?
            {
               pb_state_hau8[button_u8] -= pgm_read_byte(&ar_speed_cau8[ar_speed_u8]);
               func_pfv = (void*)pgm_read_word(&PB_Func_Tab_t[button_u8].PB_Ar_Func_pfv);
               if(func_pfv != NULL)
               {
                  func_pfv(button_u8);
               }
            } 
         }
         pb_state_hau8[button_u8]++;
      }
      else                                                           // the button is not presssed
      {
         if((pb_state_hau8[button_u8] > 0) &&                        // button was pressed before
            (pb_state_hau8[button_u8] < (pgm_read_byte(&ar_threshold_cau8[ar_thresh_u8]) - pgm_read_byte(&ar_speed_cau8[ar_speed_u8]))))
         {
            func_pfv = (void*)pgm_read_word(&PB_Func_Tab_t[button_u8].PB_Up_Func_pfv);
            if(func_pfv != NULL)
            {
               func_pfv(button_u8);
            }
         }
         pb_state_hau8[button_u8] = 0;
      }
   }
}

//-----------------------------------------------------------------------------
/*!
   \brief     void PB_String_Write_fv(uint8_t button_u8, uint8_t event_u8, 
                                      uint8_t *string_pau8)

    write push button strings into structure

   \author     Matthias Koelling
   \date       12.03.2008    

   \param[in]  button_u8 -> push button to config
               event_u8 -> event for this button to config
               *string_pau8 -> source pointer to event string 
   \param[out] none
   \return		void
*/
//-----------------------------------------------------------------------------

void PB_String_Write_fv(uint8_t button_u8, uint8_t event_u8, uint8_t *string_pau8)
{
   uint8_t i;
   uint8_t *string_pointer_pu8;

   Send_Data_String_fv("event = ", CONTROL_PC);
   Send_Char_fv(event_u8 + 48,CONTROL_PC);
   Send_Const_String_fv(sNEWLINE, CONTROL_PC);

   switch(event_u8)
   {
   case STRING_DOWN: 
      string_pointer_pu8 = &pb_strings_at[button_u8].string_dn_u8[0];
      break;    
   case STRING_UP: 
      string_pointer_pu8 = &pb_strings_at[button_u8].string_up_u8[0];
      break;    
   case STRING_AR: 
      string_pointer_pu8 = &pb_strings_at[button_u8].string_ar_u8[0];
      break;    
   default: 
      string_pointer_pu8 = NULL;
      break;    
   }
   if(string_pointer_pu8 != NULL)
   {
      for(i = 0;i < PB_STRING_LEN_DU8;i++)
      {   
         *string_pointer_pu8 = *string_pau8;
         string_pau8++;
         string_pointer_pu8++;
      }
   }
}


//-----------------------------------------------------------------------------
/*!
   \brief     void PB_Config_Write_fv(void)

    write push button config into buffer

   \author     Matthias Koelling
   \date       13.03.2008    

   \param[in]  button_u8 -> push button to config
               config_u8 -> configuration value 
   \param[out] none
   \return		void
*/
//-----------------------------------------------------------------------------


void PB_Config_Write_fv(uint8_t button_u8, uint8_t config_u8)
{
   pb_config_mau8[button_u8] = config_u8; 

}


//-----------------------------------------------------------------------------
/*!
   \brief     static void MP3_Dn_Control_fv(uint8_t button_u8)

     sends a control string to MP3 by UART depending on the button and the 
     current action (button pressed, button released, button autorepeat)

   \author     Matthias Koelling
   \date       28.05.2008    

   \param[in]  button_u8 -> push button

   \param[out] none
   \return		void
*/
//-----------------------------------------------------------------------------

static void MP3_Dn_Control_fv(uint8_t button_u8)
{
   button_gu8 = button_u8;
   Send_Data_String_fv(&pb_strings_at[button_u8].string_dn_u8[0], CONTROL_MP3);
   //Send_Data_String_fv(&pb_strings_at[button_u8].string_dn_u8[0], CONTROL_PC);
   Send_Const_String_fv(sNEWLINE, CONTROL_MP3);
   //Send_Const_String_fv(sNEWLINE, CONTROL_PC);
}


//-----------------------------------------------------------------------------
/*!
   \brief     static void MP3_Up_Control_fv(uint8_t button_u8)

     sends a control string to MP3 by UART depending on the button and the 
     current action (button pressed, button released, button autorepeat)

   \author     Matthias Koelling
   \date       28.05.2008    

   \param[in]  button_u8 -> push button

   \param[out] none
   \return		void
*/
//-----------------------------------------------------------------------------

static void MP3_Up_Control_fv(uint8_t button_u8)
{
   button_gu8 = button_u8;
   Send_Data_String_fv(&pb_strings_at[button_u8].string_up_u8[0], CONTROL_MP3);
   Send_Data_String_fv(&pb_strings_at[button_u8].string_up_u8[0], CONTROL_PC);
   Send_Const_String_fv(sNEWLINE, CONTROL_MP3);
   Send_Const_String_fv(sNEWLINE, CONTROL_PC);
}


//-----------------------------------------------------------------------------
/*!
   \brief     static void MP3_Ar_Control_fv(uint8_t button_u8)

     sends a control string to MP3 by UART depending on the button and the 
     current action (button pressed, button released, button autorepeat)

   \author     Matthias Koelling
   \date       28.05.2008    

   \param[in]  button_u8 -> push button

   \param[out] none
   \return		void
*/
//-----------------------------------------------------------------------------

static void MP3_Ar_Control_fv(uint8_t button_u8)
{
   button_gu8 = button_u8;
   Send_Data_String_fv(&pb_strings_at[button_u8].string_ar_u8[0], CONTROL_MP3);
   Send_Data_String_fv(&pb_strings_at[button_u8].string_ar_u8[0], CONTROL_PC);
   Send_Const_String_fv(sNEWLINE, CONTROL_MP3);
   Send_Const_String_fv(sNEWLINE, CONTROL_PC);
}


//-----------------------------------------------------------------------------
/*!
   \brief  static void Display_Dn_Control_fv(uint8_t button_u8)

     Doe's something depending on the button and the current action 
     (button pressed, button released, button autorepeat)

   \author     Matthias Koelling
   \date       28.05.2008    

   \param[in]  button_u8 -> push button

   \param[out] none
   \return		void
*/
//-----------------------------------------------------------------------------

static void Display_Dn_Control_fv(uint8_t button_u8)
{
   static uint8_t switch_hu8 = 0;
   
   switch_hu8++;
   Switch_Display_fv(switch_hu8 & 0x01);
}


//-----------------------------------------------------------------------------
/*!
   \brief  static void Display_Up_Control_fv(uint8_t button_u8)

     Doe's something depending on the button and the current action 
     (button pressed, button released, button autorepeat)

   \author     Matthias Koelling
   \date       28.05.2008    

   \param[in]  button_u8 -> push button

   \param[out] none
   \return		void
*/
//-----------------------------------------------------------------------------

static void Display_Up_Control_fv(uint8_t button_u8)
{
}


//-----------------------------------------------------------------------------
/*!
   \brief  static void Display_Ar_Control_fv(uint8_t button_u8)

     Doe's something depending on the button and the current action 
     (button pressed, button released, button autorepeat)

   \author     Matthias Koelling
   \date       28.05.2008    

   \param[in]  button_u8 -> push button

   \param[out] none
   \return		void
*/
//-----------------------------------------------------------------------------

static void Display_Ar_Control_fv(uint8_t button_u8)
{

}


//-----------------------------------------------------------------------------
/*!
   \brief  static void Source_Dn_Control_fv(uint8_t button_u8)

     Doe's something depending on the button and the current action 
     (button pressed, button released, button autorepeat)

   \author     Matthias Koelling
   \date       28.05.2008    

   \param[in]  button_u8 -> push button

   \param[out] none
   \return		void
*/
//-----------------------------------------------------------------------------

static void Source_Dn_Control_fv(uint8_t button_u8)
{
}


//-----------------------------------------------------------------------------
/*!
   \brief  static void Source_Up_Control_fv(uint8_t button_u8)

     Doe's something depending on the button and the current action 
     (button pressed, button released, button autorepeat)

   \author     Matthias Koelling
   \date       28.05.2008    

   \param[in]  button_u8 -> push button

   \param[out] none
   \return		void
*/
//-----------------------------------------------------------------------------

static void Source_Up_Control_fv(uint8_t button_u8)
{

}


//-----------------------------------------------------------------------------
/*!
   \brief  static void Source_Ar_Control_fv(uint8_t button_u8)

     Doe's something depending on the button and the current action 
     (button pressed, button released, button autorepeat)

   \author     Matthias Koelling
   \date       28.05.2008    

   \param[in]  button_u8 -> push button

   \param[out] none
   \return		void
*/
//-----------------------------------------------------------------------------

static void Source_Ar_Control_fv(uint8_t button_u8)
{
   static uint8_t toggle_hu8;
   if(toggle_hu8 & 0x01)
   {
      LED_POWER = ON;	
      //Send_Gateway_Command_fv(M_ENABLE);
      Send_Gateway_Command_fv(M_DISABLE);
   }
   else
   {
      LED_POWER = OFF;	
      Send_Gateway_Command_fv(M_DISABLE);
   }
   toggle_hu8++;
}

static void _2_fv(uint8_t button_u8)
{
   Init_TDA4886();
}

static void _5_fv(uint8_t button_u8)
{
   Init_OSD();
   Init_Display_fv();
}


void Display_Button_Strings_fv(void)
{
   uint8_t i;
   for(i = 0;i < BUTTON_DU8;i++)
   {
      Send_Char_fv('H',CONTROL_PC);
      Send_Char_fv(i + 48,CONTROL_PC);
      Send_Char_fv('=',CONTROL_PC);
      Send_Data_String_fv(&pb_strings_at[i].string_dn_u8[0], CONTROL_PC);
      Send_Const_String_fv(sNEWLINE, CONTROL_PC);
   }
   Send_Const_String_fv(sNEWLINE, CONTROL_PC);
   for(i = 0;i < BUTTON_DU8;i++)
   {
      Send_Char_fv('H',CONTROL_PC);
      Send_Char_fv(i + 64,CONTROL_PC);
      Send_Char_fv('=',CONTROL_PC);
      Send_Data_String_fv(&pb_strings_at[i].string_up_u8[0], CONTROL_PC);
      Send_Const_String_fv(sNEWLINE, CONTROL_PC);
   }
   Send_Const_String_fv(sNEWLINE, CONTROL_PC);
   for(i = 0;i < BUTTON_DU8;i++)
   {
      Send_Char_fv('H',CONTROL_PC);
      Send_Char_fv(i + 80,CONTROL_PC);
      Send_Char_fv('=',CONTROL_PC);
      Send_Data_String_fv(&pb_strings_at[i].string_ar_u8[0], CONTROL_PC);
      Send_Const_String_fv(sNEWLINE, CONTROL_PC);
   }
}
// eof //
