/*****************************************************************************/
/*!
\file      os.c
\brief     mini non preemptive operating system              

\implementation
  project       Multimedia Gateway
  version       0.01
  copyright     2008 Matthias Koelling 
  
  generated     Matthias Koelling 12.02.2008
  review
  released

  compiler      AVR GCC
  target        ATmega1280-16AU
\endimplementation

\history  
  Rev     Date         Author      Commentary
  0.01    27.04.2008   Koelling    adaptaption for gateway4
  0.00    12.02.2007   Koelling    generated
\endhistory  
*/
/*****************************************************************************/


/* -- Includes ------------------------------------------------------------- */

#include <avr/io.h>
#include <avr/sfr_defs.h>
#include <stdlib.h>
#include <avr/interrupt.h>
#include "hardware.h"
#include "main.h"
#include "task.h"
#include "os.h"

#define _10MS           156



static volatile uint8_t Flag_10ms_mu8;              // indicates 10ms time slice 

static void OS_Scheduler_fv(void);

//-----------------------------------------------------------------------------
/*!
   \brief     ISR(TIMER0_COMP_vect)

    timer0 output compare interrupt service routine os timing

   \author     Koelling
   \date       12.02.2008

   \param[in]  none
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

ISR(TIMER0_COMPA_vect)
{
   Flag_10ms_mu8 = TRUE;
   OCR0A = TCNT0 + _10MS;          // 10ms Intervall
}


//-----------------------------------------------------------------------------
/*!
   \brief     void Start_OS_fv(void)

    initialise os timer and start mini os

   \author     Matthias Koelling
   \date       12.02.2008

   \param[in]  none
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

void Start_OS_fv(void)
{
   OCR0A = _10MS;           // 10ms Intervall
   TCCR0A = 0x00;           // Normal port operation, OC0 disconnected
   TCCR0B = 0x05;           // prescaler = 1024 -> 1 timer clock tick is 64us long
   TIMSK0 |= _BV(OCIE0A);   // enable output compare interrupt on timer0
   sei();
   OS_Scheduler_fv();
}


//-----------------------------------------------------------------------------
/*!
   \brief     static void OS_Scheduler_fv(void)

    scheduler checks timing and starts the corresponding tasks

   \author     Matthias Koelling
   \date       12.02.2008

   \param[in]  none
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

static void OS_Scheduler_fv(void)
{
   static uint8_t flag1_50ms_u8 = FALSE;
   static uint8_t flag2_50ms_u8 = FALSE;
   static uint8_t counter_10ms_u8 = 0;       // counter for 10ms intervals
   static uint8_t counter_50ms_u8 = 0;       // counter for 50ms intervals
   static uint8_t toggle_u8;
      static uint8_t counter_hu8;

   while(1)
   {
      if(Flag_10ms_mu8 == TRUE)
      {
         if(toggle_u8 == FALSE)
         {
            toggle_u8 =TRUE;
         }
         else
         {
            toggle_u8 = FALSE;
         } 
         Flag_10ms_mu8 = FALSE;
         Task_10ms_fv();
         counter_10ms_u8++;
         switch(counter_10ms_u8)
         {
         case 1:   
            flag1_50ms_u8 = TRUE;
            break;
         case 3:
            flag2_50ms_u8 = TRUE;
            break;
         case 6:
            flag1_50ms_u8 = TRUE;
            break;
         case 8:
            flag2_50ms_u8 = TRUE;
            break;
         case 10:
            counter_10ms_u8 = 0;
            break;
         default: 
            break;
         }
         if(flag1_50ms_u8 == TRUE)
         {
            flag1_50ms_u8 = FALSE;
            Task1_50ms_fv();
         }
         if(flag2_50ms_u8 == TRUE)
         {
            flag2_50ms_u8 = FALSE;
            Task2_50ms_fv();
            counter_50ms_u8++;
            if(counter_50ms_u8 == 11)
            {
               counter_50ms_u8 = 0;
               Task_500ms_fv();
            }
         }
      }
   }
}
