/*****************************************************************************/
/*!
\file      osd.c
\brief     function for controlling osd processor stv5730
                         

\implementation
  project       Multimedia Gateway
  version       0.00
  copyright     ported from PIC16F84 onBlackBox stv5730a development board from
                G. Bjoerneseth, LA5EKA - NORWAY 
  
  generated     Matthias Koelling 26.09.2007
  review
  released

  compiler      AVR GCC
  target        ATmega128-16AU
\endimplementation

\history  
  Rev     Date         Author      Commentary
  0.00    26.09.2007   Koelling    generated
\endhistory  
*/
/*****************************************************************************/


/* -- Includes ------------------------------------------------------------- */

#include <avr/io.h>
#include <avr/sfr_defs.h>
#include <avr/eeprom.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "main.h"
#include "com.h"
#include "device.h"
#include "display.h"
#include "hardware.h"
#include "stv5730.h"

/* -- Configuration Parameters --------------------------------------------- */

/* -- Makros --------------------------------------------------------------- */

/* -- Defines -------------------------------------------------------------- */ 

#define ON		1
#define OFF		0

#define BACKGROUND_ON	1
#define BACKGROUND_OFF	0

#define	BUF_SIZE	10
#define	SCRBKGCOL 0x0E00
#define	CHRBKGCOL 0x0007
#define	CHRBORCOL 0x0038

#define COLUMS_STV5730_DU8 28
#define ROWS_STV5730_DU8   11

/* -- Types ---------------------------------------------------------------- */

/* -- Extern Global Variables ---------------------------------------------- */

/* -- Modul Global Function Prototypes ------------------------------------- */

void printascii(uint8_t color, uint8_t background,uint8_t c);
void printch(uint8_t color, uint8_t background,uint8_t c);
void puts_stv5730(uint8_t color, uint8_t background, uint8_t x, uint8_t y, const uint8_t *s);
void setPos(uint8_t x, uint8_t y);
void Reset_stv5730_fv(void);
void setDisplayedLine(uint8_t line, uint8_t Displayed);
void ClsScr(void);
void send2_stv_fv(uint16_t word_u16);
void send1_stv_fv(uint8_t byte_u8);
void send0_stv_fv(void);
void setControl(uint16_t control);
void setScrOffset(void);
//void setZoom(unsigned int z);

uint8_t scr_x=0, scr_y=0;
uint8_t scr_h=28, scr_v=24; //High text, video generator

#ifdef TEST_SCREEN
   const uint8_t CONST_Line0[]=     "          PJRC MP3          ";			//Line zoomed to 3x
   const uint8_t CONST_Line1[]=     "     Multimedia Gateway     ";
   const uint8_t CONST_Line2[]=     "                            ";
   const uint8_t CONST_Line3[]=     "         Testscreen         ";
   const uint8_t CONST_Line4[]=     "                            ";
   const uint8_t CONST_Line5[]=     "            Hallo           ";
   const uint8_t CONST_Line6[]=     "                            ";
   const uint8_t CONST_Line7[]=     "                            ";
   const uint8_t CONST_Line8[]=     "____________________________"; 
   const uint8_t CONST_Line9[]=     "                            ";
   const uint8_t CONST_Line10[]=    "                            ";
#endif
                              //           
#ifdef TV
   static uint16_t stv_control = 0b1101100010101;
#else
   static uint16_t stv_control = 0b1101001010100;
//                               0b1101001010101
#endif

static uint16_t stv_zoom    = 0b1000000000000;

static uint16_t stv_color   = 0b1000000000001;

static uint8_t SyncDetected;


static uint8_t x_offset_mu8;								 // active window x offset
static uint8_t y_offset_mu8;								 // active window y offset
static uint8_t x_end_mu8;									 // active window x end address in stv5730
static uint8_t y_end_mu8;									 // active window y end address in stv5730
static uint8_t colums_mu8;									 // active window number of colums
static uint8_t rows_mu8;									 // active window number of rows
extern  volatile uint8_t runtime_u8;
extern uint8_t  runtime_max_mu8;
 
/* -- Implementation Functions --------------------------------------------- */

/*------------------------------------------------------------------------------------*/
void Init_OSD(void)
{
	SyncDetected=0;
 
   SPSR = 0x01;      // double speed
   SPCR = 0x5D;      // CPOL = 1,CPHA = 1,CLK = 2MHz

   CSN = TRUE;      // set chip select inactive

	Reset_stv5730_fv();
   ClsScr();
}

#ifdef TEST_SCREEN

   void OSD_Test_Screen_fv(void)
   {
   	uint8_t tt;

   	for(tt=0;tt<11;tt++)
      {
   		setDisplayedLine(tt, ON);
      }
   	puts_stv5730(WHITE,BACKGROUND_ON,0,0, CONST_Line0);
   	puts_stv5730(RED,BACKGROUND_ON,0,1, CONST_Line1);
   	puts_stv5730(BLUE,BACKGROUND_ON,0,2, CONST_Line2);
   	puts_stv5730(BLUE,BACKGROUND_ON,0,3, CONST_Line3);
   	puts_stv5730(BLUE,BACKGROUND_ON,0,4, CONST_Line4);
   	puts_stv5730(BLUE,BACKGROUND_ON,0,5, CONST_Line5);
   	puts_stv5730(BLUE,BACKGROUND_ON,0,6, CONST_Line6);
   	puts_stv5730(BLUE,BACKGROUND_ON,0,7, CONST_Line7);
      puts_stv5730(BLUE,BACKGROUND_ON,0,8, CONST_Line8);
   	puts_stv5730(BLUE,BACKGROUND_ON,0,9, CONST_Line9);
   	puts_stv5730(BLUE,BACKGROUND_ON,0,10,CONST_Line10);

   }

#endif

void Set_Display_Config_fv(uint8_t x_offset_u8,uint8_t y_offset_u8, uint8_t colums_u8, uint8_t rows_u8)
{
	if(colums_u8 > COLUMS_STV5730_DU8)
	{
		x_end_mu8 = COLUMS_STV5730_DU8;
		x_offset_mu8 = 0;
	}
	else 
	{
		if((colums_u8 + x_offset_u8) > COLUMS_STV5730_DU8)
	   {
	   	x_offset_mu8 =	COLUMS_STV5730_DU8 - colums_u8;
	   	x_offset_mu8 >>= 1;
		}
		else
		{
			x_offset_mu8 = x_offset_u8;
		}
		x_end_mu8 = colums_u8 + x_offset_mu8;
	}
	if(rows_u8 > ROWS_STV5730_DU8)
	{
		y_end_mu8 = ROWS_STV5730_DU8;
		y_offset_mu8 = 0;
	}
	else 
	{
		if((rows_u8 + y_offset_u8) > ROWS_STV5730_DU8)
	   {
	   	y_offset_mu8 =	ROWS_STV5730_DU8 - rows_u8;
	   	y_offset_mu8 >>= 1;
		}
		else
		{
			y_offset_mu8 = y_offset_u8;
		}
		y_end_mu8 = rows_u8 + y_offset_mu8;
	}	
}

void Redraw_Page_STV5730_fv(uint8_t vscroll_offset_u8)
{
	uint8_t i;
	uint8_t j;
	uint8_t row_counter_u8;
	uint8_t colum_counter_u8;
	uint8_t y_pos_u8;
	
	row_counter_u8 = vscroll_offset_u8;
	colum_counter_u8 = 0;
	y_pos_u8 = y_offset_mu8;

	for(i = y_offset_mu8;i < y_end_mu8;i++)
	{
		send2_stv_fv((y_pos_u8 << 8) + x_offset_mu8);	// Set write pointer - row(y) + character number(x). Stru is 00 for page address
		for(j = x_offset_mu8;j < x_end_mu8;j++)
		{
			send2_stv_fv(DisplayBuffer_gau16[colum_counter_u8][row_counter_u8]);
   		colum_counter_u8++;
		}
		colum_counter_u8 = 0;
		y_pos_u8++;
		row_counter_u8++;
		if(row_counter_u8 == ROWS_DU8) 
      {
         row_counter_u8 = 0;
	   }
   }
}

void Clear_Page_STV5730_fv(void)
{
	uint8_t x_pos_u8;
	uint8_t y_pos_u8;

	for(y_pos_u8 = y_offset_mu8;y_pos_u8 < y_end_mu8;y_pos_u8++)
	{
		send2_stv_fv((y_pos_u8 << 8) + x_offset_mu8);	// Set write pointer - row(y) + character number(x). Stru is 00 for page address
		for(x_pos_u8 = x_offset_mu8;x_pos_u8 < x_end_mu8;x_pos_u8++)
		{
			send2_stv_fv(DATA + BE(1) + 0x0B);
		}
   }
}

void Redraw_Row_STV5730_fv(uint8_t row_u8, uint8_t vscroll_offset_u8)
{
	uint8_t j;
	uint8_t row_counter_u8;
	uint8_t colum_counter_u8;
	uint8_t y_pos_u8;
	
   row_counter_u8 = (row_u8 + vscroll_offset_u8) & 0x07;
	colum_counter_u8 = 0;
	y_pos_u8 = y_offset_mu8 + row_u8;
	
   send2_stv_fv((y_pos_u8 << 8) + x_offset_mu8);	// Set write pointer - row(y) + character number(x). Stru is 00 for page address
	for(j = x_offset_mu8;j < x_end_mu8;j++)
	{
		send2_stv_fv(DisplayBuffer_gau16[colum_counter_u8][row_counter_u8]);
   	colum_counter_u8++;
	}

}

void Clear_Row_STV5730_fv(uint8_t row_u8)
{
	uint8_t i;
	send2_stv_fv(((row_u8 + y_offset_mu8)<< 8) + x_offset_mu8);	// Set write pointer - row(y) + character number(x). Stru is 00 for page address
	for(i = x_offset_mu8;i < (x_end_mu8);i++)
	{
	   send2_stv_fv(DATA + BE(1) + 0x0B);				// clear position
	}
}

void Switch_Display_fv(uint8_t on_off_u8)
{
   uint8_t y_pos_u8;

   for(y_pos_u8 = y_offset_mu8;y_pos_u8 < y_end_mu8;y_pos_u8++)
	{
      setDisplayedLine(y_pos_u8,on_off_u8);
   }
}


/*------------------------------------------------------------------------------------*/
void setDisplayedLine(uint8_t line, uint8_t Displayed)
{
	//line is 0:10	
	send2_stv_fv(0b0000011000000+line); 			//SELECT ROW ATTRIBUTES registers (stru=11)
	if(Displayed){
	send2_stv_fv(0b1000011000000);  			//Set Row data, row enabled
	}
	else{
	send2_stv_fv(0b1000001000000);  			//Set Row data, row disabled
	}
}

/*------------------------------------------------------------------------------------*/
void Reset_stv5730_fv(void)
{
	send2_stv_fv(0x3000);							
	send2_stv_fv(0x3000);	
	send2_stv_fv(0x00DB);					    //SEND RESET
	send2_stv_fv(0x1000);  
 	//send2_stv_fv(WRITE_POINTER(0,3,12)); //SELECT ZOOM REGISTER (1ST CONTROL REG)		address 12
   send2_stv_fv(0b0000011001100);  //SELECT ZOOM REGISTER (1ST CONTROL REG)		address 12
	send2_stv_fv(stv_zoom); 	//SEND ZOOM DATA					address 12
	send2_stv_fv(stv_color); 	//SEND COLOR DATA					address 13
	send2_stv_fv(stv_control);	//Send control data					address 14
	send2_stv_fv(0x1000 + (scr_v << 6) + scr_h); //SEND POSITION DATA			address 15
	#ifdef TV
      send2_stv_fv(0b1000100010100); 	//SEND MODE DATA					address 16
   #else
      send2_stv_fv(0b1000100010100); 	//SEND MODE DATA					address 16
 //                  0b1000100100100
   #endif
				//(M2 indicates mute high on missing sync, M3 is 32lines)
}
/*------------------------------------------------------------------------------------*/
void ClsScr(void)
{
   uint16_t j;
	//send2_stv_fv(WRITE_POINTER(0,3,0));
   send2_stv_fv(0b0000011000000); 			//SELECT ROW ATTRIBUTES, Start at ROW 0
	for(j=0;j<=10;j++)			//SEND ROW ATTRIB DATA, Chars at current row displayed, and have border 
		send2_stv_fv(0b1000011000000);  	//ROW number increments, 0-10.
						//Erase screen with space to prevent initial garbage
	send2_stv_fv(0b0000000000000);			//Select PAGE ROW 0
	for(j=0;j<=307;j++)
 		send2_stv_fv(0b0001011100001011);	//Write incremental data, character d11 (' ')

	send2_stv_fv(0b0000000000000);			//Select PAGE ROW 0

}
/*------------------------------------------------------------------------------------*/

void setControl(uint16_t control)
{
	send2_stv_fv(0b0000011001110); 
	send2_stv_fv(control);
}


/*------------------------------------------------------------------------------------*/
void setPos(uint8_t x, uint8_t y)
{
	if(	(x<28) && (y<11))
	{
		scr_x=x; scr_y=y;
	}
}

/*------------------------------------------------------------------------------------*/
void puts_stv5730(uint8_t color, uint8_t background, uint8_t x, uint8_t y, const uint8_t *s)
{
	setPos(x, y);
	while(*s)
		printascii(color,background,*s++);
}

/*------------------------------------------------------------------------------------*/
void printascii(uint8_t color, uint8_t background,uint8_t c)
{
  if(c==' ') {printch(color,background,0x0B);}
  else if(c=='/') {printch(color,background,0x28);}
  else if(c=='|') {printch(color,background,110);}	//sat dish
  else if(c==':') {printch(color,background,0x26);}
  else if(c=='-') {printch(color,background,0x0A);}//den andre er nr 0d100
  else if(c=='_') {printch(color,background,114);}
  else if(c=='.') {printch(color,background,0x27);} 
	else if((c>=65) && (c<95)) {printch(color,background,c-53);}   
	else if((c>=97) && (c<123)) {printch(color,background,c-55);}
	else if((c>=48) && (c<60)) {printch(color,background,c-48);}
}

/*------------------------------------------------------------------------------------*/
void printch(uint8_t color, uint8_t background,uint8_t c)
{
unsigned int OutData;
	//Write pointer is Buf[11:8], Stru[7:6],DontCare[5] Depl[4:0]
	send2_stv_fv((scr_y << 8) + scr_x);	//Set write pointer - row(y) + character number(x). Stru is 00 for page address
	scr_x++;
	if (scr_x >= 28)
	{
		scr_x=0;
  	scr_y++;
		if(scr_y >= 11)
		{ scr_x=27; scr_y=10;}
	}
	//[BE,11][R,10][G,9][B,8][BK,7][Charcode, 6:0]
	//send2_stv_fv(0b1 0 100 0 0000000+c);
	OutData=0b1000000000000;
	if(background){
	OutData|=0b0100000000000;	
	}
	OutData=OutData|(color<<8);
	//send2_stv_fv((DATA | BE(1) | RGB(4) | BK(0))+c);
   send2_stv_fv(OutData+c); 
}

void putc_stv5730(uint8_t x_pos_u8, uint8_t y_pos_u8, uint16_t char_u16)
{
   x_pos_u8 += x_offset_mu8;
   y_pos_u8 += y_offset_mu8; 	
   if((x_pos_u8 < COLUMS_STV5730_DU8) && (y_pos_u8 < ROWS_STV5730_DU8))
   {
      send2_stv_fv((y_pos_u8 << 8) + x_pos_u8);	//Set write pointer - row(y) + character number(x). Stru is 00 for page address
	   send2_stv_fv(char_u16); 
   }
}

/*------------------------------------------------------------------------------------*/
void setScrOffset(void)
{
	send2_stv_fv(0b0000011001111); 
	send2_stv_fv(0x1000 + (scr_v << 6) + scr_h);
}

/*------------------------------------------------------------------------------------*/
/*
void setZoom(unsigned int zoom)
{
	send2_stv_fv(0b0000011001100); //SELECT ZOOM REGISTER (1ST CONTROL REG)	address 12
	send2_stv_fv(zoom); 
}
*/

/*------------------------------------------------------------------------------------*/

//This function writes 2 bytes to the stv device
void send2_stv_fv(uint16_t word_u16)
{
   volatile uint8_t byte0;
   volatile uint8_t byte1;

   CSN = FALSE; // chip select low

   byte0 = (uint8_t)(word_u16 & 0x00FF);
   byte1 = (uint8_t)(word_u16 >> 8);
  
   SPDR = byte1;
 
   #ifndef SIMU   
      loop_until_bit_is_set(SPSR,SPIF);
   #endif
 
   SPDR = byte0;
   #ifndef SIMU   
      loop_until_bit_is_set(SPSR,SPIF);
   #endif
   
   byte0 = SPDR; // to reset SPIF flag after last transmission
 
   CSN = TRUE; // chip select high
   _delay_loop_1(10);
}

//This function writes 1 byte to the stv device
void send1_stv_fv(uint8_t byte_u8)
{
   uint8_t byte0;
   uint8_t byte1;

   CSN = FALSE; // chip select low
   
   SPDR = byte_u8;
#ifndef SIMU   
   loop_until_bit_is_set(SPSR,SPIF);
#endif   
   byte0 = 0; // dummy
   byte1 = 0; // dummy

   CSN = TRUE; // chip select high
}

//This function writes 0 bytes to the stv device
void send0_stv_fv(void)
{
   uint8_t byte0;
   uint8_t byte1;

   CSN = FALSE; // chip select low
   
   byte0 = 0; // dummy
   byte1 = 0; // dummy

   CSN = TRUE; // chip select high
}

