/*****************************************************************************/
/*!
\file      twi.c
\brief     driver and handling functions for TWI (I2C)              

\implementation
  project       Multimedia Gateway
  version       0.00
  copyright     
  
  generated     Matthias Koelling 26.04.2009
  review
  released

  compiler      AVR GCC
  target        ATmega1280-16AU
\endimplementation

\history  
  Rev     Date         Author      Commentary
  0.00    26.04.2009   Koelling    generated
\endhistory  
*/
/*****************************************************************************/

/* -- Includes ------------------------------------------------------------- */

#include <avr/io.h>
#include <avr/pgmspace.h>
#include "hardware.h"
#include "com.h"
#include "printf.h"
#include "main.h"
#include "twi.h"

/* -- Configuration Parameters --------------------------------------------- */

/* -- Makros --------------------------------------------------------------- */

/* -- Defines -------------------------------------------------------------- */ 

/* -- Types ---------------------------------------------------------------- */

/* -- Modul Global Function Prototypes ------------------------------------- */

 static void Twi_wait_int(void);

/* -- Module Global Variables ---------------------------------------------- */

static uint8_t Error;        // error status 
static uint8_t Twi_control;  // control register local copy
static uint8_t Twi_status;   // status register local copy

/* -- Extern Global Variables ---------------------------------------------- */

/* -- Implementation Functions --------------------------------------------- */

//-----------------------------------------------------------------------------
/*!
   \brief     Init_TWI

    initialise TWI interface

   \author     Koelling
   \date       26.04.2009

   \param[in]  void
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

void Init_TWI(void)
{
   //TWCR = _BV(TWEN); // enable TWI
   TWSR = 0;         // set prescaler to 0 
   TWBR = 72;        // 16000000 / (16 + (2 * 72 * 4^0)) = 100 kbit
}


//-----------------------------------------------------------------------------
/*!
   \brief     Twi_send_byte

    send 1 byte to a slave

   \author     Koelling
   \date       26.04.2009

   \param[in]  slave_u8 -> slave address
   \param[in]  sub_u8 -> sub address
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

void Twi_send_byte(uint8_t slave_u8,uint8_t sub_u8,uint8_t data_u8)
{
   Error = 0;                               // clear error 

    // start condition 
   TWCR = _BV(TWINT) | _BV(TWSTA) | _BV(TWEN);  

    // wait for TWINT 
   Twi_wait_int();

    // when access to the bus is possible send slave address 
   if((Twi_status == 0x08) || (Twi_status == 0x10))
   {
    
      TWDR = slave_u8 & 0xFE;               // slave address + write 
      TWCR = 0b10000100;                    // clear TWINT, send slave address 

      // wait for TWINT 
      Twi_wait_int();
      
      //printf("ok1\r\n");

      // response from slave 
      if((Twi_status == 0x18) || (Twi_status == 0x20))
      {
         TWDR = sub_u8;                     // sub address 
         TWCR = 0b10000100;                 // clear TWINT, send sub address 

         // wait for  WINT
         Twi_wait_int();
         
         if((Twi_status == 0x28) || (Twi_status == 0x30))
         { 
            TWDR = data_u8;                 // data 
            TWCR = 0b10000100;              // clear TWINT, send data 
   
            // wait for TWINT 
            Twi_wait_int();
              // data sent out
            if((Twi_status == 0x28) || (Twi_status == 0x30))
            {
               Error = 0;                    // no error 
               #ifdef DEBUG
                  printf("TWI_OK\r\n");
               #endif
            }
            else
            {
               Error = Twi_status;           // error 
               #ifdef DEBUG
                  printf("\n\rF1");
               #endif
            }
         }
         else                                // no slave 
         {
            Error = Twi_status;              // error 
            #ifdef DEBUG
               Send_Const_String_fv(PSTR("\n\rF2"),CONTROL_PC);
            #endif
      
         }
      }
      else                                   // no slave 
      {
         Error = Twi_status;                 // error 
         #ifdef DEBUG
            Send_Const_String_fv(PSTR("\n\rF3"),CONTROL_PC);
         #endif
      }
      
      // always send STOP condition undepending on the current status 
      TWCR = 0b10010100;                     // clear TWINT, send STOP
      // after sending STOP TWINT will not be set, 
      // you need not / must not wait for it ! 
   }
   else                                      // bus is not free 
   {
      TWCR = 0b10000100;                     // clear TWINT, free bus 
      Error = Twi_status;                    // error 
      Send_Const_String_fv(PSTR("\n\rF4"),CONTROL_PC);
   }
   //printf("Status = %x\n\r",Twi_status );
}


//-----------------------------------------------------------------------------
/*!
   \brief     Twi_wait_int

    wait until TWINT is set and read status register

   \author     Koelling
   \date       26.04.2009

   \param[in]  void
   \param[out] none
   \return     void
*/
//-----------------------------------------------------------------------------

static void Twi_wait_int(void)
{
   #ifndef SIMU   
      loop_until_bit_is_set(TWCR,TWINT);
   #endif
   Twi_status = TWSR & 0xF8;      // read status 
}

//-----------------------------------------------------------------------------
/*!
   \brief     Twi_send_byte

    send 1 byte to a slave

   \author     Koelling
   \date       26.04.2009

   \param[in]  slave_u8 -> slave address
   \param[in]  sub_u8   -> sub address
   \param[out] none
   \return     data
*/
//-----------------------------------------------------------------------------

uint8_t Twi_read_byte(uint8_t slave_u8, uint8_t sub_u8)
{
   uint8_t twi_read_byte_u8;
   Error = 0;                                // clear error

   twi_read_byte_u8 = 0;                                    

   // start condition 
   TWCR = 0b10100100;                        // TWINT  

   // wait for TWINT 
   Twi_wait_int();

    // when access to the bus is possible send slave address 
   if((Twi_status == 0x08) || (Twi_status == 0x10))
   {
      
      TWDR = slave_u8 | 0x01;                // slave adresse + Read 
      TWCR = 0b10000100;                     // clear TWINT, send slave address 

      // wait for TWINT
      Twi_wait_int();

      if((Twi_status == 0x18) || (Twi_status == 0x20))
      {   
      
         TWDR = sub_u8;                      // sub address
         TWCR = 0b10000100;                  // TWINT lschen, Byte senden 

         // wait for TWINT 
         Twi_wait_int();
         
         // response from slave 
         if((Twi_status == 0x48) || (Twi_status == 0x40))
         {
            TWCR = 0b10000100;               // clear TWINT
            // don't send ACK (TWEA = 0), because we want to read only one byte 
            
            // wait for TWINT 
            Twi_wait_int();

            // one Byte was received 
            if((Twi_status == 0x58) || (Twi_status == 0x50))
            {
               twi_read_byte_u8 = TWDR;      // read data 
               Error = 0;                    // no error 
               Send_Const_String_fv(PSTR("\nOk"),CONTROL_PC);
            }
            else
            {
               Error = Twi_status;           // error 
               Send_Const_String_fv(PSTR("\nF5"),CONTROL_PC);
             }
         }
         else
         {
            Error = Twi_status;              // error 
            Send_Const_String_fv(PSTR("\nF6"),CONTROL_PC);
         }
      }
      else                                   // no slave 
      {
         Error = Twi_status;                 // error 
         Send_Const_String_fv(PSTR("\nF7"),CONTROL_PC);
      }
      // always send STOP condition undepending on the current status 
      TWCR = 0b10010100;                     // clear TWINT, send STOP
      // after sending STOP TWINT will not be set, 
      // you need not / must not wait for it ! 
   }
   else                                      // bus is not free  
   {
      TWCR = 0b10000100;                     // clear TWINT, free bus
      Error = Twi_status;                    // error 
      Send_Const_String_fv(PSTR("\nF8"),CONTROL_PC);
   }
   return twi_read_byte_u8;
}

